@@ -96,6 +96,13 @@ find_matching_sent_article (const Article * article)
 			if (!strcmp (body_1, body_2))
 				retval = compare;
 		}
+		else
+		if (body_1==NULL)
+		{
+			/* article's body is NULL, meaning it hasn't been
+			   read yet. Risk it and cancel this one */
+			retval = compare;
+		}
 		g_free (body_1);
 		g_free (body_2);
 	}
@@ -111,13 +118,47 @@ find_matching_sent_article (const Article * article)
 static gint
 article_cancel_run (TaskFuncPtr* task, gpointer user_data)
 {
-        int status = nntp_cancel (STATUS_ITEM(task),
-	                          ARTICLE(user_data),
-	                          TASK(task)->sock);
+	Article * article = ARTICLE(user_data);
+        int       status = nntp_cancel (STATUS_ITEM(task),
+			                article,
+	                                TASK(task)->sock);
 
 	if (status == TASK_SUCCESS)
 	{
-		/* FIXME: add something to the article to denote that it's been cancelled */
+		gchar        * subject = g_strdup (article_get_subject (article));
+		gchar        * message_id = g_strdup (article_get_message_id (article));
+
+#if 0
+		/* FIXME: this doesn't seem to work: group_get_article_by_message_id */
+		/* doesn't return the article. Not sure why.                         */
+
+		const gchar  * newsgroups = article_get_extra_header (article, HEADER_NEWSGROUPS);
+		gchar       ** groups = g_strsplit (newsgroups, ",", 0);
+		guint          i;
+
+		for (i=0; groups[i]; i++) {
+
+			Group     * group = server_get_named_group (TASK(task)->server, groups[i]);
+		      
+			group_ref_articles (group, STATUS_ITEM(task));
+
+			article = (Article *) group_get_article_by_message_id (group, message_id);
+			if (article) {
+				group_remove_article (group, article);
+			}
+
+			group_unref_articles (group, STATUS_ITEM(task));
+		}
+
+		g_strfreev (groups);
+
+#endif
+		log_add_va (LOG_INFO|LOG_URGENT, _("Article %s (%s) cancelled"),
+			subject, message_id);
+
+		g_free (subject);
+		g_free (message_id);
+
 	}
 
 	return 0;
@@ -141,14 +182,11 @@ article_cancel (Article * article)
 {
 	Server * server;
 	Article * ours;
-	const gchar * newsgroups;
 
 	/* sanity clause */
 	g_return_if_fail (article != NULL);
 	server = article->group==NULL ? NULL : article->group->server;
 	g_return_if_fail (server != NULL);
-	newsgroups = article_get_header (article, HEADER_NEWSGROUPS);
-	g_return_if_fail (is_nonempty_string (newsgroups));
 
 	ours = find_matching_sent_article (article);
 	if (ours == NULL)
@@ -316,7 +354,7 @@ open_article (const Article * a)
 
 	g_return_if_fail (article_is_valid(a));
 
-	files = article_get_header (a, PAN_HEADER_FILENAME);
+	files = article_get_extra_header (a, PAN_HEADER_FILENAME);
 	while ((file = get_next_token_str (files, '\n', &files))) {
 		open_outside_file (file);
 		g_free (file);
@@ -358,7 +396,7 @@ decode_impl (Article * a,
 		save_attachments = TRUE;
 
 	/* if user is opening the files & we already have them... */
-	files = article_get_header (a, PAN_HEADER_FILENAME);
+	files = article_get_extra_header (a, PAN_HEADER_FILENAME);
 	if (open_attachments && is_nonempty_string(files)) {
 		open_article (a);
 		debug_exit ("decode_impl");
