@@ -24,18 +24,41 @@
 #define ARTICLE_TOOLBAR_H
 
 #include <gtk/gtkwidget.h>
+#include <pan/filters/filter.h>
+#include <pan/filters/filter-phrase.h>
+#include <pan/articlelist.h> /* for FilterShow */
 
-typedef enum
-{
-	ARTICLE_FILTER_SUBJECT				= 1,
-	ARTICLE_FILTER_AUTHOR				= 2
-}
-ArticleFilterMode;
-
+/**
+***
+**/
 
 GtkWidget*  article_toolbar_new          (void);
 
-void        article_toolbar_get_filter   (gchar             ** filter_allocme,
-                                          ArticleFilterMode  * type);
+void        article_toolbar_get_filter   (Filter            ** filter_newme,
+                                          gulong             * filter_bits,
+                                          FilterShow         * filtershow_setme);
+
+void        article_toolbar_set_filter   (const gchar        * filter_name,
+                                          gulong               filter_bits,
+                                          FilterShow           filtershow_setme);
+
+/**
+***  Events
+**/
+
+typedef struct
+{
+	Filter * filter;
+	gulong filter_bits;
+	FilterShow filter_show;
+}
+ArticleToolbarCallbackStruct;
+
+/**
+ * @call_obj: const ArticleToolbarCallbackStruct*
+ * @call_arg: NULL
+ */
+PanCallback*  article_toolbar_get_filter_changed_callback (void);
+
 
 #endif
