@@ -24,8 +24,10 @@
 #include <errno.h>
 #include <math.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/time.h>
+#include <unistd.h>
 
 #include <glib.h>
 
@@ -46,50 +48,61 @@
 ****
 ***/
 
-static gboolean
+static ServerGroupsType
 file_grouplist_097_style_exists (const Server * server)
 {
-	gboolean retval = FALSE;
+	gchar fname [PATH_MAX];
+	ServerGroupsType retval = 0;
 
-	g_return_val_if_fail (server!=NULL, FALSE);
+	/* sanity clause */
+	g_return_val_if_fail (server_is_valid(server), FALSE);
 	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
 
-	if (!retval)
-	{
-		gchar * fname = g_strdup_printf ("%s%c%s_sub.idx", data_dir, G_DIR_SEPARATOR, server->name);
-       		retval = file_exists (fname);
-		g_free (fname);
-	}
-	if (!retval)
-	{
-		gchar * fname = g_strdup_printf ("%s%c%s_unsub.idx", data_dir, G_DIR_SEPARATOR, server->name);
-       		retval = file_exists (fname);
-		g_free (fname);
-	}
+	/* check for sub */
+	g_snprintf (fname, sizeof(fname), "%s%c%s_sub.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server));
+	if (file_exists (fname))
+		retval |= SERVER_GROUPS_SUBSCRIBED;
 
+	/* check for unsub */
+	g_snprintf (fname, sizeof(fname), "%s%c%s_unsub.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server));
+	if (file_exists (fname))
+		retval |= SERVER_GROUPS_UNSUBSCRIBED;
+
+	/* return results */
 	return retval;
 }
+
 static gboolean
 file_grouplist_096_style_exists (const Server * server)
 {
+	gchar fname [PATH_MAX];
 	gboolean retval;
-	gchar * fname;
 
-	g_return_val_if_fail (server!=NULL, FALSE);
+	g_return_val_if_fail (server_is_valid(server), FALSE);
 	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
        
-	fname = g_strdup_printf ("%s%c%s.idx", data_dir, G_DIR_SEPARATOR, server->name);
+	g_snprintf (fname, sizeof(fname), "%s%c%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server));
        	retval = file_exists (fname);
-	g_free (fname);
 
 	return retval;
 }
-gboolean
+
+ServerGroupsType
 file_grouplist_exists (const Server* server)
 {
-	g_return_val_if_fail (server!=NULL, FALSE);
+	ServerGroupsType retval = 0;
 
-	return file_grouplist_097_style_exists(server) || file_grouplist_096_style_exists(server);
+	/* sanity clause */
+	g_return_val_if_fail (server_is_valid(server), 0);
+
+	/* first check the new style... */
+	retval = file_grouplist_097_style_exists (server);
+
+	/* fall back on the old style... */
+	if (!retval && file_grouplist_096_style_exists (server))
+		retval = SERVER_GROUPS_ALL;
+
+	return retval;
 }
 
 
@@ -113,14 +126,15 @@ read_group_data (Server        * server,
 	debug_enter ("read_group_data");
 
 	/* sanity clause */
-	g_return_val_if_fail (server!=NULL, 0);
+	g_return_val_if_fail (server_is_valid(server), 0);
 	g_return_val_if_fail (dat_str!=NULL, 0);
 	g_return_val_if_fail (idx!=NULL, 0);
 
 	version = get_next_token_int (march_idx, '\n', &march_idx);
 	groups = g_ptr_array_new ();
 	if (version==1 || version==2 || version==3 || version==4 || 
-	    version==5 || version==6)
+	    version==5 || version==6 || version==7 || version==8 ||
+	    version==9 || version==10)
 	{
 		long i;
 		glong qty = get_next_token_long (march_idx, '\n', &march_idx);
@@ -184,12 +198,38 @@ read_group_data (Server        * server,
 					g->identity_name = g_strdup (dat_str+l);
 			}
 
-			g->flags            = (gint8)   get_next_token_int   (march_idx, '\n', &march_idx);
-			g->filter_bits      = (guint16) get_next_token_int   (march_idx, '\n', &march_idx);
-			g->sort_style       = (gint8)   get_next_token_int   (march_idx, '\n', &march_idx);
-			g->article_low      = (gulong)  get_next_token_ulong (march_idx, '\n', &march_idx);
+			g->flags = (gint8)   get_next_token_int   (march_idx, '\n', &march_idx);
+
+			if (version>=8) {
+				g->filter_show = get_next_token_ulong (march_idx, '\n', &march_idx);
+			}
+
+			g->filter_bits = (guint32) get_next_token_ulong (march_idx, '\n', &march_idx);
+
+			/* old sort style */
+			if (version >= 10)
+				g->old_sort_style = get_next_token_int (march_idx, '\n', &march_idx);
+
+			g->new_sort_style = (gint8) get_next_token_int (march_idx, '\n', &march_idx);
+
+			/* compare sort styles */
+			if (version >= 10)
+				if (abs(g->old_sort_style) == abs(g->new_sort_style))
+					g->old_sort_style = 0;
+
+			g->article_low = (gulong) get_next_token_ulong (march_idx, '\n', &march_idx);
+
+			if (version>=9)
+				g->loaded_since_last_fetch = get_next_token_int (march_idx, '\n', &march_idx) != 0;
+			else
+				g->loaded_since_last_fetch = FALSE;
+
 			g->article_high     = (gulong)  get_next_token_ulong (march_idx, '\n', &march_idx);
 
+			/* filter_bits got widened from guint16 to guint32 in version 7 */
+			if (version<=6)  
+				g->filter_bits |= 0xFFFF8000;
+
 			/* article_high_old added in version 5 */
 			if (version>=5)
 				g->article_high_old = (gulong)  get_next_token_ulong (march_idx, '\n', &march_idx);
@@ -235,14 +275,14 @@ read_group_data (Server        * server,
 	{
 		log_add_va (LOG_ERROR, _("Unsupported data version %d for `%s' data file.\nAre you running an old version of Pan by accident?"),
 			version,
-			server->name);
+			server_get_name (server));
 	}
 
 	group_qty = groups->len;
 
 	if (groups->len != 0) {
 		GPtrArray * unused = g_ptr_array_new ();
-		server_add_groups (server, (Group**)groups->pdata, groups->len, NULL, unused);
+		server_init_groups (server, (Group**)groups->pdata, groups->len, NULL, unused);
 		pan_g_ptr_array_foreach (unused, (GFunc)pan_object_unref, NULL);
 		g_ptr_array_free (unused, TRUE);
 	}
@@ -261,25 +301,24 @@ file_grouplist_load_file (Server * server, StatusItem * status, const gchar * fi
 	debug_enter ("file_grouplist_load_file");
 
 	/* sanity clause */
-	g_return_val_if_fail (server!=NULL, 0);
-	g_return_val_if_fail (is_nonempty_string(server->name), 0);
-	g_return_val_if_fail (is_nonempty_string(filemarker), 0);
+	g_return_val_if_fail (server_is_valid (server), 0);
+	g_return_val_if_fail (filemarker!=NULL, 0);
 	g_return_val_if_fail (is_nonempty_string(data_dir), 0);
 
 	/* open the index file */
 	if (1) {
-		gchar * path = g_strdup_printf ("%s%c%s%s.idx", data_dir, G_DIR_SEPARATOR, server->name, filemarker);
+		gchar path [PATH_MAX];
+		g_snprintf (path, sizeof(path), "%s%c%s%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
 		pan_normalize_filename_inplace (path);
 		idx = read_file (path);
-		g_free (path);
 	}
 
 	/* open the data file */
 	if (1) {
-		gchar * path = g_strdup_printf ("%s%c%s%s.dat", data_dir, G_DIR_SEPARATOR, server->name, filemarker);
+		gchar path [PATH_MAX];
+		g_snprintf (path, sizeof(path), "%s%c%s%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
 		pan_normalize_filename_inplace (path);
 		dat = read_file (path);
-		g_free (path);
 	}
 
 	/* read the groups */
@@ -302,12 +341,12 @@ file_grouplist_load (Server * server, ServerGroupsType group_type, StatusItem *
 	int group_qty = 0;
 	debug_enter ("file_grouplist_load");
 
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid(server));
 	g_return_if_fail (group_type!=0);
 
 	gettimeofday (&start, NULL);
 
-	if (file_grouplist_097_style_exists(server))
+	if (file_grouplist_097_style_exists (server))
 	{
 		/* load from single files */
 		if (group_type & SERVER_GROUPS_SUBSCRIBED)
@@ -333,7 +372,7 @@ file_grouplist_load (Server * server, ServerGroupsType group_type, StatusItem *
 		diff += (finish.tv_usec - start.tv_usec)/1000000.0;
 		log_add_va (LOG_INFO, _("Loaded %d groups for server `%s' in %.1f seconds (%.0f groups/sec)"),
 			group_qty,
-			server->name,
+			server_get_name (server),
 			diff,
 			group_qty/(fabs(diff)<0.001?0.001:diff));
 	}
@@ -364,18 +403,18 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 
 	/* Write the group information... */
 	pos = 0;
-	fprintf (idx_fp, "6\n"); /* file format version number */
+	fprintf (idx_fp, "10\n"); /* file format version number */
 	fprintf (idx_fp, "%u\n", groups->len); /* number of groups */
 	for (i=0; i!=groups->len; ++i)
 	{
 		Group * group = GROUP(g_ptr_array_index (groups, i));
 
 		gchar * pch = group->_newsrc==NULL
-			? g_strdup("")
+			? NULL
 			: newsrc_get_read_str (group_get_newsrc(group));
 
 		gchar * purged = group->_purged==NULL
-			? g_strdup("")
+			? NULL
 			: newsrc_get_read_str (group_get_purged(group));
 
 		/* write the group name */
@@ -383,19 +422,22 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 		const long read_idx = get_string_offset (NULL, NULL, group->readable_name, dat_fp, &pos);
 		const long desc_idx = get_string_offset (NULL, NULL, group->description, dat_fp, &pos);
 		const long path_idx = get_string_offset (NULL, NULL, group->download_dir, dat_fp, &pos);
-		const long news_idx = get_string_offset (NULL, NULL, pch, dat_fp, &pos);
-		const long dead_idx = get_string_offset (NULL, NULL, purged, dat_fp, &pos);
+		const long news_idx = get_string_offset (NULL, NULL, pch?pch:"", dat_fp, &pos);
+		const long dead_idx = get_string_offset (NULL, NULL, purged?purged:"", dat_fp, &pos);
 		const long filt_idx = get_string_offset (NULL, NULL, group->filter_name, dat_fp, &pos);
 		const long iden_idx = get_string_offset (NULL, NULL, group->identity_name, dat_fp, &pos);
 
 		/* write the non-string fields. */
 		fprintf (idx_fp,
 		         "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n"
-		         "%u\n" "%u\n" "%d\n" "%lu\n" "%lu\n" "%lu\n" "%d\n" "%d\n" "%c\n",
+		         "%u\n" "%lu\n" "%lu\n" "%d\n" "%d\n" "%d\n" "%lu\n" "%lu\n" "%lu\n" "%d\n" "%d\n" "%c\n",
 		         name_idx, read_idx, desc_idx, path_idx, news_idx, dead_idx, filt_idx, iden_idx,
 		         (unsigned)(group->flags & ~GROUP_NEW),
-		         (unsigned)group->filter_bits,
-		         (int)group->sort_style,
+		         group->filter_show,
+		         (gulong)group->filter_bits,
+		         (int)group->old_sort_style,
+		         (int)group->new_sort_style,
+		         (int)(group->loaded_since_last_fetch ? 1 : 0),
 		         (unsigned long)group->article_low,
 		         (unsigned long)group->article_high,
 			 (unsigned long)group->article_high_old,
@@ -406,15 +448,17 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 		/* let the user know what's going on */
 		if (status != NULL) {
 			status_item_emit_next_step (status);
-			if (!(i % 512))
+			if (!(i % 1024))
 				status_item_emit_status_va (status,
 					_("Saved %d of %u groups"),
 					i, groups->len);
 		}
 
 		/* cleanup */
-		g_free (pch);
-		g_free (purged);
+		if (pch != NULL)
+			g_free (pch);
+		if (purged != NULL)
+			g_free (purged);
 	}
 
 	debug_exit ("write_groups");
@@ -423,46 +467,56 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 static void
 save_groups_to_file (Server * server, GPtrArray * groups, gchar * filemarker, StatusItem * status)
 {
-	gchar * pch;
-	FILE * idx_fp = NULL;
-	FILE * dat_fp = NULL;
-	gchar * idx_path = pan_make_temp (&idx_fp);
-	gchar * dat_path = pan_make_temp (&dat_fp);
 	debug_enter ("save_groups_to_file");
 
 	/* sanity clause */
-	g_return_if_fail (server!=NULL);
-	g_return_if_fail (is_nonempty_string(server->name));
+	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail (groups!=NULL);
 	g_return_if_fail (0<=groups->len);
 	g_return_if_fail (is_nonempty_string(filemarker));
 
-	/* make sure that we were able to create the temp files */
-	if (idx_fp==NULL || dat_fp==NULL) {
+	if (groups->len == 0)
+	{
+		gchar path [PATH_MAX];
+		g_snprintf (path, PATH_MAX, "%s%c%s_%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
+		unlink (path);
+		g_snprintf (path, PATH_MAX, "%s%c%s_%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
+		unlink (path);
+	}
+	else
+	{
+		gchar path [PATH_MAX];
+		FILE * idx_fp = NULL;
+		FILE * dat_fp = NULL;
+		gchar * idx_path = pan_make_temp (&idx_fp);
+		gchar * dat_path = pan_make_temp (&dat_fp);
+
+		/* make sure that we were able to create the temp files */
+		if (idx_fp==NULL || dat_fp==NULL)
+		{
+			fclose (idx_fp);
+			fclose (dat_fp);
+			g_free (idx_path);
+			g_free (dat_path);
+			g_return_if_fail (0);
+		}
+
+		/* save the groups */
+		write_groups (idx_fp, dat_fp, groups, status);
 		fclose (idx_fp);
 		fclose (dat_fp);
+
+		/* move the tmp files to the real thing */
+		g_snprintf (path, PATH_MAX, "%s%c%s_%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
+		pan_normalize_filename_inplace (path);
+		pan_rename (idx_path, path);
 		g_free (idx_path);
+		g_snprintf (path, PATH_MAX, "%s%c%s_%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
+		pan_normalize_filename_inplace (path);
+		pan_rename (dat_path, path);
 		g_free (dat_path);
-		g_return_if_fail (0);
+		debug_exit ("save_groups_to_file");
 	}
-
-	/* save the groups */
-	write_groups (idx_fp, dat_fp, groups, status);
-	fclose (idx_fp);
-	fclose (dat_fp);
-
-	/* move the tmp files to the real thing */
-	pch = g_strdup_printf ("%s%c%s_%s.idx", data_dir, G_DIR_SEPARATOR, server->name, filemarker);
-	pan_normalize_filename_inplace (pch);
-	pan_rename (idx_path, pch);
-	g_free (idx_path);
-	g_free (pch);
-	pch = g_strdup_printf ("%s%c%s_%s.dat", data_dir, G_DIR_SEPARATOR, server->name, filemarker);
-	pan_normalize_filename_inplace (pch);
-	pan_rename (dat_path, pch);
-	g_free (dat_path);
-	g_free (pch);
-	debug_exit ("save_groups_to_file");
 }
 
 void
@@ -473,8 +527,7 @@ file_grouplist_save (Server * server, ServerGroupsType group_type, StatusItem *
 	debug_enter ("file_grouplist_save");
 
 	/* sanity clause */
-	g_return_if_fail (server!=NULL);
-	g_return_if_fail (server->_groups!=NULL);
+	g_return_if_fail (server_is_valid(server));
 	g_return_if_fail (is_nonempty_string(data_dir));
 	g_return_if_fail ((group_type|SERVER_GROUPS_SUBSCRIBED)||(group_type|SERVER_GROUPS_UNSUBSCRIBED));
 
@@ -510,7 +563,7 @@ file_grouplist_save (Server * server, ServerGroupsType group_type, StatusItem *
 		diff += (finish.tv_usec - start.tv_usec)/1000000.0;
 		log_add_va (LOG_INFO, _("Saved %d groups in \"%s\" in %.1f seconds (%.0f groups/sec)"),
 			save_qty,
-			server->name,
+			server_get_name (server),
 			diff,
 			save_qty/(fabs(diff)<0.001?0.001:diff));
 	}
