@@ -57,22 +57,19 @@ file_headers_load_group (Group * group, StatusItem * status)
 	GArray *dat, *idx;
 	debug_enter ("file_headers_load_group");
 
-	g_return_val_if_fail (group!=NULL, FALSE);
-	g_return_val_if_fail (group->server!=NULL, FALSE);
-	g_return_val_if_fail (is_nonempty_string (group->name), FALSE);
-	g_return_val_if_fail (is_nonempty_string (group->server->name), FALSE);
+	g_return_val_if_fail (group_is_valid (group), FALSE);
 	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
 
 	if (status!=NULL)
 		status_item_emit_status_va (status, _("Loading %s"), group_get_readable_name(group));
 
 	/* open the index file */
-	path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
+	path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (group->server), G_DIR_SEPARATOR, group->name);
 	idx = read_file (path);
 	g_free (path);
 
 	/* open the data file */
-	path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
+	path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (group->server), G_DIR_SEPARATOR, group->name);
 	dat = read_file (path);
 	g_free (path);
 
@@ -228,16 +225,16 @@ get_mbox_filename (const Group * folder)
 	debug_enter ("get_mbox_filename");
 
 	g_return_val_if_fail (folder!=NULL, NULL);
-	g_return_val_if_fail (folder->server!=NULL, NULL);
+	g_return_val_if_fail (server_is_valid (folder->server), NULL);
 	g_return_val_if_fail (group_is_folder(folder), NULL);
 	g_return_val_if_fail (is_nonempty_string(data_dir), NULL);
-	g_return_val_if_fail (is_nonempty_string(folder->server->name), NULL);
 
-	retval = g_strdup_printf ("%s%c%s%c%s.mbox", data_dir,
-	                                             G_DIR_SEPARATOR,
-	                                             folder->server->name,
-	                                             G_DIR_SEPARATOR,
-	                                             folder->name);
+	retval = g_strdup_printf ("%s%c%s%c%s.mbox", 
+			data_dir,
+	                G_DIR_SEPARATOR,
+	                server_get_name (folder->server),
+	                G_DIR_SEPARATOR,
+	                folder->name);
 
 	debug_exit ("get_mbox_filename");
 	return retval;
@@ -419,7 +416,7 @@ file_headers_save_folder (Group * folder, StatusItem * status)
 
 	/* make sure the directory exists */
 	if (ok) {
-		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, folder->server->name);
+		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, server_get_name (folder->server));
 		if (!directory_check (pch)) {
 			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
 			ok = FALSE;
@@ -523,9 +520,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 	gboolean ok = TRUE;
 	debug_enter ("file_headers_save_group");
 
-	g_return_val_if_fail (group!=NULL, FALSE);
-	g_return_val_if_fail (group->server!=NULL, FALSE);
-	g_return_val_if_fail (group->_articles!=NULL, FALSE);
+	g_return_val_if_fail (group_is_valid(group), FALSE);
 	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
 
 	/* if nothing to save */
@@ -541,7 +536,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 	**/
 
 	if (1) {
-		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, group->server->name);
+		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, server_get_name (group->server));
 		if (!directory_check (pch))
 			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
 		g_free (pch);
@@ -551,7 +546,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 	/* open index file */
 	idx_path = g_strdup_printf ("%s%c%s%c%s.idx.tmp",
 		data_dir, G_DIR_SEPARATOR,
-		group->server->name, G_DIR_SEPARATOR,
+		server_get_name (group->server), G_DIR_SEPARATOR,
 		group->name);
 	idx_fp = fopen (idx_path, "w+");
 	if (idx_fp == NULL) {
@@ -563,7 +558,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 	/* open data file */
 	dat_path = g_strdup_printf ("%s%c%s%c%s.dat.tmp",
 		data_dir, G_DIR_SEPARATOR,
-		group->server->name, G_DIR_SEPARATOR,
+		server_get_name (group->server), G_DIR_SEPARATOR,
 		group->name);
 	dat_fp = fopen (dat_path, "w+");
 	if (dat_fp == NULL) {
@@ -667,7 +662,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 	{
 		pch = g_strdup_printf ("%s%c%s%c%s.idx",
 			data_dir, G_DIR_SEPARATOR,
-			group->server->name, G_DIR_SEPARATOR,
+			server_get_name (group->server), G_DIR_SEPARATOR,
 			group->name);
 		ok = pan_rename (idx_path, pch);
 		g_free (idx_path);
@@ -678,7 +673,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 	{
 		pch = g_strdup_printf ("%s%c%s%c%s.dat",
 			data_dir, G_DIR_SEPARATOR,
-			group->server->name, G_DIR_SEPARATOR,
+			server_get_name (group->server), G_DIR_SEPARATOR,
 			group->name);
 		ok = pan_rename (dat_path, pch);
 		g_free (dat_path);
@@ -753,7 +748,7 @@ void
 file_headers_save (Group * group, StatusItem * status)
 {
 	debug_enter ("file_headers_save");
-	g_return_if_fail (group!=NULL);
+	g_return_if_fail (group_is_valid(group));
 
 	group_ref_articles (group, status);
 	file_headers_save_noref (group, status);
@@ -767,8 +762,7 @@ file_headers_destroy (const Group * group)
 {
 	debug_enter ("file_headers_destroy");
 
-	g_return_if_fail (group!=NULL);
-	g_return_if_fail (group->server!=NULL);
+	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (is_nonempty_string(data_dir));
 
 	if (group_is_folder(group))
@@ -781,11 +775,11 @@ file_headers_destroy (const Group * group)
 	{
 		gchar * path;
 
-		path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
+		path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (group->server), G_DIR_SEPARATOR, group->name);
 		remove (path);
 		g_free (path);
 
-		path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, group->server->name, G_DIR_SEPARATOR, group->name);
+		path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (group->server), G_DIR_SEPARATOR, group->name);
 		remove (path);
 		g_free (path);
 	}
