@@ -39,8 +39,6 @@
 #include <pan/base/server.h>
 
 static void fire_groups_changed (Group **, gint qty);
-static void fire_group_cleared (Group *);
-static void fire_group_marked_read (Group *, gboolean read);
 
 /**
 ***
@@ -51,7 +49,7 @@ group_new (Server * server, const char * name)
 {
 	Group * group;
 	
-	g_return_val_if_fail (server!=NULL, NULL);
+	g_return_val_if_fail (server_is_valid (server), NULL);
 	pan_warn_if_fail (is_nonempty_string(name));
 
 	group = server_alloc_new_group (server);
@@ -68,10 +66,11 @@ group_constructor (Group                * g,
 
 	debug1 (DEBUG_PAN_OBJECT, "group_constructor: %p", g);
 	g->article_mutex = g_mutex_new ();
+	g->loaded_since_last_fetch = FALSE;
 	g->articles_dirty = FALSE;
-	g->articles_threading_dirty = FALSE;
 	g->flags = (gint8)0;
-	g->sort_style = (gint8)0;
+	g->old_sort_style = (gint8)0;
+	g->new_sort_style = (gint8)0;
 	g->filter_bits = ~((guint16)STATE_FILTER_IGNORED);
 	g->filter_show = FILTER_SHOW_MATCHES_AND_REFERENCES;
 	g->article_qty = (gint32)0;
@@ -93,6 +92,7 @@ group_constructor (Group                * g,
 	g->_newsrc = NULL;
 	g->_purged = NULL;
 	g->_articles = NULL;
+	g->_deleted_articles = NULL;
 	g->_one_big_chunk = NULL;
 }
 
@@ -126,22 +126,27 @@ group_destructor (Group * o)
 		pan_object_unref (PAN_OBJECT(g->_purged));
 
 	/* articles */
-	g_mutex_free (g->article_mutex);
 	pan_warn_if_fail (g->_articles_refcount >= 0);
 	replace_gstr (&g->_one_big_chunk, NULL);
-	if (g->_article_chunk != NULL) {
-		g_mem_chunk_destroy (g->_article_chunk);
-		g->_article_chunk = NULL;
-	}
-	if (g->chunk != NULL) {
-		g_string_chunk_free (g->chunk);
-		g->chunk = NULL;
-	}
 	if (g->_articles != NULL) {
 		g_hash_table_foreach (g->_articles, destroy_articles_ghfunc, NULL);
 		g_hash_table_destroy (g->_articles);
 		g->_articles = NULL;
 	}
+	if (g->_deleted_articles != NULL) {
+		pan_g_ptr_array_foreach (g->_deleted_articles, (GFunc)article_destructor, NULL);
+		g_ptr_array_free (g->_deleted_articles, TRUE);
+		g->_deleted_articles = NULL;
+	}
+	if (g->chunk != NULL) {
+		g_string_chunk_free (g->chunk);
+		g->chunk = NULL;
+	}
+	if (g->_article_chunk != NULL) {
+		memchunk_destroy (g->_article_chunk);
+		g->_article_chunk = NULL;
+	}
+	g_mutex_free (g->article_mutex);
 }
 
 /***
@@ -174,6 +179,7 @@ groups_set_subscribed (Group ** groups, gint qty, gboolean subscribed)
 {
 	gint i;
 	ServerGroupsType type;
+	Server * server = NULL;
 
 	/* sanity clause */
 	g_return_if_fail (groups != NULL);
@@ -186,7 +192,10 @@ groups_set_subscribed (Group ** groups, gint qty, gboolean subscribed)
 		Group * group = groups[i];
 
 		if (group->server != NULL)
-			server_set_group_type_dirty (group->server, type);
+		{
+			server = group->server;
+			server_set_group_type_dirty (server, type);
+		}
 
 		if (subscribed)
 			group_set_flags (group, group->flags|GROUP_SUBSCRIBED);
@@ -194,6 +203,9 @@ groups_set_subscribed (Group ** groups, gint qty, gboolean subscribed)
 			group_set_flags (group, group->flags&~GROUP_SUBSCRIBED);
 	}
 
+	if (server != NULL)
+		server_save_grouplist_if_dirty (server, NULL);
+
 	fire_groups_changed (groups, qty);
 }
 
@@ -262,7 +274,9 @@ group_set_sort_style (Group * group, int sort_style)
 {
 	g_return_if_fail (group != NULL);
 
-	group->sort_style = sort_style;
+	if (abs(sort_style) != abs(group->new_sort_style))
+		group->old_sort_style = group->new_sort_style;
+	group->new_sort_style = sort_style;
 	group_set_dirty (group);
 }
 
@@ -340,14 +354,42 @@ void
 group_mark_all_read (Group * g, gboolean read)
 {
 	Newsrc * newsrc;
+	GPtrArray * articles;
+	debug_enter ("group_mark_all_read");
 
+	/* sanity clause */
 	g_return_if_fail (group_is_valid(g));
 
+	/* ensure the articles are loaded */
+	group_ref_articles (g, NULL);
+
+	/* mark all the articles as read.  use articles_set_read for this 
+	   because that updates all the unread_children fields, marks 
+	   crossposts as read, etc... */
+
+	g_mutex_lock (g->article_mutex);
+	articles = group_get_article_array (g);
+	if (articles->len > 0)
+		articles_set_read  ((Article**)articles->pdata, articles->len, TRUE);
+	g_ptr_array_free (articles, TRUE);
+	g_mutex_unlock (g->article_mutex);
+
+	/**
+	 * Wipe the entire newsgroup's article range as read.
+	 * This reduces the group's newsrc to a single range "low-high"
+	 * instead of thousands of smaller ranges.
+	 */
 	newsrc = group_get_newsrc (g);
 	newsrc_set_group_range (newsrc, g->article_low, g->article_high);
 	newsrc_mark_all (newsrc, read);
+
+	/* update group counts */
 	group_set_article_read_qty (g, (read ? g->article_qty : 0));
-	fire_group_marked_read (g, read);
+
+	/* unref the articles */
+	group_unref_articles (g, NULL);
+
+	debug_exit ("group_mark_all_read");
 }
 
 gboolean
@@ -370,7 +412,7 @@ gboolean
 group_is_valid (const Group * group)
 {
 	g_return_val_if_fail (group!=NULL, FALSE);
-	g_return_val_if_fail (group->server!=NULL, FALSE);
+	g_return_val_if_fail (server_is_valid (group->server), FALSE);
 	g_return_val_if_fail (group->article_mutex!=NULL, FALSE);
 	g_return_val_if_fail (is_nonempty_string(group->name), FALSE);
 
@@ -417,32 +459,26 @@ group_set_is_folder (Group * group, gboolean folder)
 void
 group_empty (Group * group, gboolean clear_counts)
 {
+	GPtrArray * a;
+
+	/* sanity clause */
 	g_return_if_fail (group_is_valid(group));
 
-	fire_group_cleared (group);
+	/* remove the articles from disk */
 	file_headers_destroy (group);
 
-	if (group->_articles != NULL) {
-		g_hash_table_foreach (group->_articles, destroy_articles_ghfunc, NULL);
-		g_hash_table_destroy (group->_articles);
-		group->_articles = NULL;
-	}
-	if (group->_article_chunk != NULL) {
-		g_mem_chunk_destroy (group->_article_chunk);
-		group->_article_chunk = NULL;
-	}
+	/* remove the articles from memory */
+	a = group_get_article_array (group);
+	group_remove_articles (group, a);
+	g_ptr_array_free (a, TRUE);
+
+	/* reset the counters */
 	group_set_article_qty (group, 0);
 	group_set_article_read_qty (group, 0);
 
 	/* if user wants to clear the range, do that too */
 	if (clear_counts)
 		group_set_article_range (group, 0, 0);
-
-	/* flush the flags */
-	if (group->flags & GROUP_SUBSCRIBED)
-		group_set_flags (group, GROUP_SUBSCRIBED);
-	else
-		group_set_flags (group, 0);
 }
 
 /**
@@ -476,46 +512,19 @@ group_add_article (Group * group, Article * article)
 	debug_exit ("group_add_article");
 }
 
- 
-void
-group_add_articles_remove_unused (Group            * group,
-                                  GPtrArray        * articles,
-                                  StatusItem       * status)
-{
-	GPtrArray * ignored;
-	GPtrArray * used;
-
-	/* entry assertions */
-	g_return_if_fail (group_is_valid(group));
-	g_return_if_fail (articles != NULL);
-	g_return_if_fail (articles->len != 0);
-
-	/* add the articles */
-	ignored  = g_ptr_array_new ();
-	used = g_ptr_array_new ();
-	group_add_articles (group, articles, status, used, ignored);
-
-	/* destroy unused articles & remove from the incoming array */
-	pan_g_ptr_array_assign (articles, used->pdata, used->len);
-
-	/* cleanup */
-	g_ptr_array_free (ignored, TRUE);
-	g_ptr_array_free (used, TRUE);
-}
-
-void
-group_add_articles (Group             * group,
-                    const GPtrArray   * articles,
-                    StatusItem        * status,
-                    GPtrArray         * used,
-                    GPtrArray         * ignored)
+static void
+group_add_articles_impl (Group             * group,
+                         const GPtrArray   * articles,
+                         StatusItem        * status,
+                         GPtrArray         * used,
+                         GPtrArray         * ignored,
+                         gboolean            lock_group)
 {
 	gint old_qty;
 	gint new_read_qty;
 	guint i;
 	gulong low;
 	gulong high;
-	gboolean tmp_ref;
 	GPtrArray * tmp_used;
 	GHashTable * ours;
 	Newsrc * newsrc;
@@ -528,13 +537,7 @@ group_add_articles (Group             * group,
 	g_return_if_fail (articles!=NULL);
 	if (!articles->len) return;
 	g_return_if_fail (articles->len>0);
-
-	/* warn the caller if they're doing something inefficient by adding
-	   articles to a group whose refcount is zero.  The old articles in
-	   this group have to be loaded too; otherwise they'll be overwritten
-	   when we save this group to disk without them. */
-	if ((tmp_ref = !group->_articles_refcount))
-		group_ref_articles (group, status);
+	g_return_if_fail (articles_are_valid ((const Article**)articles->pdata, articles->len));
 
 	/**
 	***  Remove the articles from their old groups.
@@ -596,7 +599,10 @@ group_add_articles (Group             * group,
 	purged = group_get_purged (group);
 	pan_g_ptr_array_reserve (tmp_used, articles->len);
 
-	g_mutex_lock (group->article_mutex);
+	if (lock_group) {
+		g_mutex_lock (group->article_mutex);
+	}
+
 	ours = group_get_articles (group);
 	old_qty = (int) g_hash_table_size (ours);
 	for (i=0; i!=articles->len; ++i)
@@ -623,7 +629,24 @@ group_add_articles (Group             * group,
 			g_ptr_array_add (ignored, a);
 		}
 	}
-	g_mutex_unlock (group->article_mutex);
+
+	/**
+	***  Rethread the articles
+	**/
+
+	if (tmp_used->len != 0) {
+		GPtrArray * a = group_get_article_array (group);
+		thread_articles (a);
+		g_ptr_array_free (a, TRUE);
+	}
+
+	/**
+	***  Unlock the article mutex
+	**/
+
+	if (lock_group) {
+		g_mutex_unlock (group->article_mutex);
+	}
 
 	/**
 	***  Now update our stats if articles were added
@@ -636,7 +659,6 @@ group_add_articles (Group             * group,
 			group_set_article_range (group, low, high);
 
 		/* update the group article stats */
-		group->articles_threading_dirty = TRUE;
 		group_set_article_qty (group, g_hash_table_size(ours));
 		if (old_qty != 0)
 			new_read_qty += group->article_read_qty;
@@ -652,11 +674,64 @@ group_add_articles (Group             * group,
 
 	/* cleanup */
 	g_ptr_array_free (tmp_used, TRUE);
-	if (tmp_ref)
-		group_unref_articles (group, status);
+		
 	debug_exit ("group_add_articles");
 }
 
+static void
+group_add_articles_remove_unused_impl (Group            * group,
+                                       GPtrArray        * articles,
+                                       StatusItem       * status,
+				       gboolean           lock)
+{
+	GPtrArray * ignored;
+	GPtrArray * used;
+
+	/* entry assertions */
+	g_return_if_fail (group_is_valid(group));
+	g_return_if_fail (articles != NULL);
+	g_return_if_fail (articles->len != 0);
+
+	/* add the articles */
+	ignored  = g_ptr_array_new ();
+	used = g_ptr_array_new ();
+	group_add_articles_impl (group, articles, status, used, ignored, lock);
+
+	/* destroy unused articles & remove from the incoming array */
+	pan_g_ptr_array_assign (articles, used->pdata, used->len);
+	pan_g_ptr_array_foreach (ignored, (GFunc)article_destructor, NULL);
+
+	/* cleanup */
+	g_ptr_array_free (ignored, TRUE);
+	g_ptr_array_free (used, TRUE);
+}
+
+void
+group_add_articles_remove_unused (Group            * group,
+                                  GPtrArray        * articles,
+                                  StatusItem       * status)
+{
+	group_add_articles_remove_unused_impl (group, articles, status, TRUE);
+}
+
+void
+group_init_articles (Group * group,
+                     GPtrArray * articles,
+                     StatusItem * status)
+{
+	group_add_articles_remove_unused_impl (group, articles, status, FALSE);
+}
+
+void
+group_add_articles (Group             * group,
+                    const GPtrArray   * articles,
+                    StatusItem        * status,
+                    GPtrArray         * used,
+                    GPtrArray         * ignored)
+{
+	group_add_articles_impl (group, articles, status, used, ignored, TRUE);
+}
+
 void
 group_set_articles_dirty (Group * group)
 {
@@ -665,36 +740,12 @@ group_set_articles_dirty (Group * group)
 	group->articles_dirty = TRUE;
 }
 
-void
-group_thread_if_needed (Group * group, StatusItem * status)
-{
-	g_return_if_fail (group_is_valid(group));
-
-	debug_enter ("group_thread_if_needed");
-
-	if (group->articles_threading_dirty && group->_articles!=NULL && g_hash_table_size(group->_articles)!=0)
-	{
-		GPtrArray * a;
-
-		g_mutex_lock (group->article_mutex);
-		a = group_get_article_array (group);
-		thread_articles (a, status);
-		check_multipart_articles (a);
-		group->articles_threading_dirty = FALSE;
-		g_mutex_unlock (group->article_mutex);
-
-		g_ptr_array_free (a, TRUE);
-	}
-
-	debug_exit ("group_thread_if_needed");
-}
-
- 
 gpointer
 group_get_article_by_message_id (Group * group, const gchar * message_id)
 {
 	g_return_val_if_fail (group_is_valid(group), NULL);
 	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
+	g_return_val_if_fail (*message_id=='<', NULL);
 
 	return g_hash_table_lookup (group_get_articles(group), message_id);
 }
@@ -713,44 +764,13 @@ group_remove_article (Group * group, Article * article)
 	g_ptr_array_free (a, TRUE);
 }
 
-static void
-group_isolate_article (Group * g, Article * a)
-{
-	GSList *l = NULL;
-	Article * parent;
-
-	g_return_if_fail (group_is_valid (g));
-	g_return_if_fail (article_is_valid (a));
-	g_return_if_fail (a->group == g);
-	
-	parent = a->parent;
-
-	g_mutex_lock (g->article_mutex);
-
-	if (parent)
-		parent->threads = g_slist_remove (parent->threads, a);
-
-	for (l=a->threads; l!=NULL; l=l->next) {
-		Article *child = ARTICLE(l->data);
-		child->parent = parent;
-		if (parent)
-			parent->threads = g_slist_append (parent->threads, child);
-	}
-
-	g_slist_free (a->threads);
-	a->threads = NULL;
-	a->parent = NULL;
-
-	g_mutex_unlock (g->article_mutex);
-}
-
 void
 group_remove_articles (Group * group, const GPtrArray * articles)
 {
 	guint i;
 	int removed_read;
-	int removed_total;
 	GPtrArray * tmp_remove;
+	GPtrArray * removed;
 	GHashTable * ours;
 	debug_enter ("group_remove_articles");
 
@@ -768,21 +788,14 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 	sort_articles ((Article**)tmp_remove->pdata, tmp_remove->len, ARTICLE_SORT_MSG_ID, TRUE);
 	pan_callback_call (group_get_articles_removed_callback(), group, tmp_remove);
 
-	/* is this the last gasp for these articles? */
-	for (i=0; i!=tmp_remove->len; ++i) {
-		Article * article = ARTICLE(g_ptr_array_index(tmp_remove,i));
-		if (PAN_OBJECT(article)->ref_count == 1) /* about to die */
-			group_isolate_article (article->group, article);
-	}
-
 	/**
 	***  Try to remove the articles
 	**/
 
 	g_mutex_lock (group->article_mutex);
 	ours = group_get_articles (group);
-	removed_total = 0;
 	removed_read = 0;
+	removed = g_ptr_array_new ();
 	for (i=0; i!=tmp_remove->len; ++i)
 	{
 		Article * a = ARTICLE(g_ptr_array_index(tmp_remove,i));
@@ -793,28 +806,72 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 		{
 			g_hash_table_remove (ours, message_id);
 
-			++removed_total;
+			/* add to the 'removed' array */
+			g_ptr_array_add (removed, our_a);
+
+			/* count how many removed articles were read */
 			if (article_is_read (our_a))
 				++removed_read;
 		}
 	}
+
+	/**
+	***  Rethread the affected articles
+	**/
+
+	if (removed->len)
+	{
+		GPtrArray * rethread_a = article_get_unique_threads (removed, GET_WHOLE_THREAD);
+		GHashTable * rethread_hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+		/* add removed articles to the 'deleted' list */
+		if (group->_deleted_articles == NULL)
+			group->_deleted_articles = g_ptr_array_new ();
+		pan_g_ptr_array_append (group->_deleted_articles, removed->pdata, removed->len);
+
+		/* build a hash table of the articles that need to be rethreaded */
+		for (i=0; i!=rethread_a->len; ++i) {
+			Article * a = ARTICLE(g_ptr_array_index(rethread_a, i));
+			g_hash_table_insert (rethread_hash, (gpointer)a->message_id, a);
+		}
+		/* remove the deleted articles from this rethread list */
+		for (i=0; i!=removed->len; ++i) {
+			Article * a = ARTICLE(g_ptr_array_index(removed, i));
+			g_hash_table_remove (rethread_hash, a->message_id);
+		}
+
+		/* rethread the affected articles */
+		if (g_hash_table_size(rethread_hash) > 0) {
+			pan_hash_to_ptr_array (rethread_hash, rethread_a);
+			thread_articles (rethread_a);
+		}
+
+		/* cleanup */
+		g_hash_table_destroy (rethread_hash);
+		g_ptr_array_free (rethread_a, TRUE);
+	}
+
+	/**
+	***  Unlock the lock
+	**/
+
 	g_mutex_unlock (group->article_mutex);
 
 	/**
 	***  Update the stats
 	**/
 
-	if (removed_total != 0)
+	if (removed->len != 0)
 	{
 		/* update the articles stats */
-		group->articles_threading_dirty = TRUE;
-		group->article_qty -= removed_total;
+		group->article_qty -= removed->len;
 		group->article_read_qty -= removed_read;
 		group_set_articles_dirty (group);
 		fire_groups_changed (&group, 1);
 	}
 
 	/* cleanup */
+	g_ptr_array_free (removed, TRUE);
 	g_ptr_array_free (tmp_remove, TRUE);
 	debug_exit ("group_remove_articles");
 }
@@ -861,6 +918,8 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 ****
 ***/
 
+static GStaticMutex article_refcount_mutex = G_STATIC_MUTEX_INIT;
+
 void
 group_ref_articles (Group * group, StatusItem * status)
 {
@@ -870,12 +929,16 @@ group_ref_articles (Group * group, StatusItem * status)
 	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (group->_articles_refcount >= 0);
 
-	/* if we're up from zero, load the headers */
+	g_static_mutex_lock (&article_refcount_mutex);
 	if (++group->_articles_refcount == 1)
+	{
+		g_mutex_lock (group->article_mutex);
 		file_headers_load (group, status);
-/*g_message ("group [%s] article refcount up to %d", group->name, group->_articles_refcount);*/
+		group_set_article_qty (group, g_hash_table_size(group->_articles));
+		g_mutex_unlock (group->article_mutex);
+	}
+	g_static_mutex_unlock (&article_refcount_mutex);
 
-	group_set_article_qty (group, g_hash_table_size(group->_articles));
 	debug_exit ("group_ref_articles");
 }
 
@@ -884,20 +947,30 @@ group_unref_articles (Group * group, StatusItem * status)
 {
 	debug_enter ("group_unref_articles");
 
+	/* sanity clause */
 	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (group->_articles_refcount > 0);
 
-	if (!--group->_articles_refcount)
+	/* unref */
+	g_static_mutex_lock (&article_refcount_mutex);
+	if (group->_articles_refcount == 1)
 	{
-		file_headers_save (group, status);
+		g_mutex_lock (group->article_mutex);
+		file_headers_save_noref (group, NULL);
+		server_save_grouplist_if_dirty (group->server, NULL);
 
 		if (group->_articles!=NULL) {
 		       	g_hash_table_foreach (group->_articles, destroy_articles_ghfunc, NULL);
 			g_hash_table_destroy (group->_articles);
 			group->_articles = NULL;
 		}
+		if (group->_deleted_articles != NULL) {
+			pan_g_ptr_array_foreach (group->_deleted_articles, (GFunc)article_destructor, NULL);
+			g_ptr_array_free (group->_deleted_articles, TRUE);
+			group->_deleted_articles = NULL;
+		}
 		if (group->_article_chunk != NULL) {
-			g_mem_chunk_destroy (group->_article_chunk);
+			memchunk_destroy (group->_article_chunk);
 			group->_article_chunk = NULL;
 		}
 
@@ -907,8 +980,12 @@ group_unref_articles (Group * group, StatusItem * status)
 			g_string_chunk_free (group->chunk);
 			group->chunk = NULL;
 		}
+		g_mutex_unlock (group->article_mutex);
 	}
+	--group->_articles_refcount;
+	g_static_mutex_unlock (&article_refcount_mutex);
 
+	/* cleanup */
 	debug_exit ("group_unref_articles");
 }
 
@@ -1098,9 +1175,9 @@ group_alloc_new_article (Group * group)
 	g_return_val_if_fail (group_is_valid(group), NULL);
 
 	if (group->_article_chunk == NULL)
-		group->_article_chunk = g_mem_chunk_create (Article, 200, G_ALLOC_ONLY);
+		group->_article_chunk = memchunk_new (sizeof(Article), 200, FALSE);
 
-	a =  g_chunk_new0 (Article, group->_article_chunk);
+	a = (Article*) memchunk_alloc (group->_article_chunk);
 	a->group = group;
 	return a;
 }
@@ -1109,7 +1186,8 @@ GHashTable*
 group_get_articles (Group * group)
 {
 	g_return_val_if_fail (group_is_valid(group), NULL);
- 
+
+	/* FIXME: shouldn't this load the articles ??? */
 	if (group->_articles == NULL)
 		group->_articles = g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -1129,6 +1207,7 @@ group_get_article_array (Group * group)
 	/* populate retval */
 	hash = group_get_articles (group);
 	pan_hash_to_ptr_array (hash, retval);
+	pan_warn_if_fail (articles_are_valid ((const Article**)retval->pdata, retval->len));
 
 	return retval;
 }
@@ -1174,37 +1253,3 @@ group_get_articles_removed_callback (void)
 	if (cb==NULL) cb = pan_callback_new ();
 	return cb;
 }
-
-PanCallback*
-group_get_group_marked_read_callback (void)
-{
-	static PanCallback * cb = NULL;
-	if (cb==NULL) cb = pan_callback_new ();
-	return cb;
-}
-
-static void
-fire_group_marked_read (Group * group, gboolean read)
-{
-	g_return_if_fail (group!=NULL);
-
-	pan_callback_call (group_get_group_marked_read_callback(),
-	                   group,
-	                   GINT_TO_POINTER(read));
-}
-
-PanCallback*
-group_get_group_emptied_callback (void)
-{
-	static PanCallback * cb = NULL;
-	if (cb==NULL) cb = pan_callback_new ();
-	return cb;
-}
-
-static void
-fire_group_cleared (Group * group)
-{
-	g_return_if_fail (group!=NULL);
-
-	pan_callback_call (group_get_group_emptied_callback(), group, NULL);
-}
