@@ -39,9 +39,9 @@
 ***/
 
 /* null-safe strstr */
-gchar*
-pan_strstr (const gchar * s1,
-            const gchar * s2)
+char*
+pan_strstr (const char * s1,
+            const char * s2)
 {
 	g_return_val_if_fail (s1!=NULL, NULL);
 	g_return_val_if_fail (s2!=NULL, NULL);
@@ -51,8 +51,8 @@ pan_strstr (const gchar * s1,
 
 /* null-safe strcmp.  NULLs < strings. */
 gint 
-pan_strcmp (const gchar * a,
-            const gchar * b)
+pan_strcmp (const char * a,
+            const char * b)
 {
 	gint retval;
 
@@ -68,12 +68,12 @@ pan_strcmp (const gchar * a,
 	return retval;
 }
 
-gchar*
+char*
 pan_stristr (const char * string, const char * pattern)
 {
 	size_t slen;
 	size_t plen;
-	char * start = (gchar*) string;
+	char * start = (char*) string;
 
 	g_return_val_if_fail (string!=NULL, NULL);
 	g_return_val_if_fail (pattern!=NULL, NULL);
@@ -83,8 +83,8 @@ pan_stristr (const char * string, const char * pattern)
 
 	for (; slen>=plen; ++start, --slen)
 	{
-		gchar * sptr;
-		gchar * pptr;
+		char * sptr;
+		char * pptr;
 
 		/* find start of pattern in string */
 		while (toupper(*start) != toupper(*pattern))
@@ -114,8 +114,8 @@ pan_stristr (const char * string, const char * pattern)
 }
 
 gboolean
-string_ends_with (const gchar * str,
-                  const gchar * end)
+string_ends_with (const char * str,
+                  const char * end)
 {
 	size_t str_len;
 	size_t end_len;
@@ -160,10 +160,10 @@ pan_substitute (const char * original,
 	size_t rlen;		/* length of replace */
 	size_t tlen;		/* length of target (predicted) */
 	gint i;
-	const gchar * o;
-	const gchar * pchar;
-	gchar * t;
-	gchar * retval = NULL;
+	const char * o;
+	const char * pchar;
+	char * t;
+	char * retval = NULL;
 
 	g_return_val_if_fail (original!=NULL, NULL);
 	g_return_val_if_fail (*original!='\0', NULL);
@@ -327,7 +327,7 @@ pan_g_string_replace (GString       * string,
                       const char    * search,
 		      const char    * replace)
 {
-	const gchar * pch;
+	const char * pch;
 	size_t slen;
 	size_t rlen;
 	size_t offset;
@@ -361,18 +361,26 @@ pan_g_string_strstrip (GString * string)
 }
 
 void
-pan_g_string_append_len  (GString       * str,
-                          const gchar   * appendme,
-                          guint           len)
+pan_g_string_append_len  (GString      * str,
+                          const char   * appendme,
+                          guint          len)
 {
-	gchar * pch;
-
 	g_return_if_fail (str!=NULL);
 	g_return_if_fail (appendme!=NULL);
 
-	pch = g_strndup (appendme, len);
-	g_string_append (str, pch);
-	g_free (pch);
+	if (len < 1024)
+	{
+		char buf[1024];
+		memcpy (buf, appendme, len);
+		buf[len] = '\0';
+		g_string_append (str, buf);
+	}
+	else
+	{
+		char * pch = g_strndup (appendme, len);
+		g_string_append (str, pch);
+		g_free (pch);
+	}
 }
 
 /***
@@ -398,14 +406,14 @@ skip_next_token (const char   * pch,
 	}
 }
  
-const gchar*
-get_next_token_range (const gchar   * pch,
-                      const gchar     delimiter,
-                      const gchar  ** setme_next_token,
-                      const gchar  ** setme_start,
-                      gint          * setme_len)
+const char*
+get_next_token_run (const char   * pch,
+                    const char     delimiter,
+                    const char  ** setme_next_token,
+                    const char  ** setme_start,
+                    int          * setme_len)
 {
-	const gchar * start = pch;
+	const char * start = pch;
 
 	if (pch!=NULL)
 		while (*pch && *pch!=delimiter)
@@ -413,7 +421,9 @@ get_next_token_range (const gchar   * pch,
 
 	*setme_start = start;
 	*setme_len = pch!=NULL ? pch-start : 0;
-	*setme_next_token = pch!=NULL && *pch==delimiter ? pch+1 : pch;
+	if (setme_next_token != NULL)
+		*setme_next_token = pch!=NULL && *pch==delimiter ? pch+1 : pch;
+
 	return is_nonempty_string(start) ? start : NULL;
 }
 
@@ -427,8 +437,8 @@ get_next_token_long (const char   * pch,
 
 	if (is_nonempty_string(pch))
 	{
-		const gchar * m;
-		const gchar * end = NULL;
+		const char * m;
+		const char * end = NULL;
 
 		/* find the end of this token */
 		skip_next_token (pch, delimiter, &end);
@@ -439,7 +449,7 @@ get_next_token_long (const char   * pch,
 		   otherwise if isspace(token) is true, atoi may walk
 		   right into the next token and give a wrong retval */
 		for (m=pch; m!=end; ++m) {
-			if (*m=='-' || *m=='+' || isdigit((int)*m)) {
+			if (*m=='-' || *m=='+' || isdigit((guchar)*m)) {
 				retval = atol (m);
 				break;
 			}
@@ -458,8 +468,8 @@ get_next_token_ulong (const char   * pch,
 
 	if (is_nonempty_string(pch))
 	{
-		const gchar * m;
-		const gchar * end = NULL;
+		const char * m;
+		const char * end = NULL;
 
 		/* find the end of this token */
 		skip_next_token (pch, delimiter, &end);
@@ -470,7 +480,7 @@ get_next_token_ulong (const char   * pch,
 		   otherwise if isspace(token) is true, strtoul may walk
 		   right into the next token and give a wrong retval */
 		for (m=pch; m!=end; ++m) {
-			if (*m=='-' || *m=='+' || isdigit((int)*m)) {
+			if (*m=='-' || *m=='+' || isdigit((guchar)*m)) {
 				retval = strtoul (m, NULL, 10);
 				break;
 			}
@@ -489,8 +499,8 @@ get_next_token_int (const char   * pch,
 
 	if (is_nonempty_string(pch))
 	{
-		const gchar * m;
-		const gchar * end = NULL;
+		const char * m;
+		const char * end = NULL;
 
 		/* find the end of this token */
 		skip_next_token (pch, delimiter, &end);
@@ -501,7 +511,7 @@ get_next_token_int (const char   * pch,
 		   otherwise if isspace(token) is true, atoi may walk
 		   right into the next token and give a wrong retval */
 		for (m=pch; m!=end; ++m) {
-			if (*m=='-' || *m=='+' || isdigit((int)*m)) {
+			if (*m=='-' || *m=='+' || isdigit((guchar)*m)) {
 				retval = atoi(m);
 				break;
 			}
@@ -546,17 +556,13 @@ get_next_token_g_str (const char   * pch,
 	while (is_nonempty_string(end) && *end!=delimiter)
 		++end;
 
-	if (end != pch)
-	{
-		gchar * tmp = g_strndup (pch, end-pch);
-		g_string_assign (setme, tmp);
-		g_free (tmp);
-		retval = TRUE;
-	}
-	else
-	{
-		g_string_truncate (setme, 0);
+	g_string_truncate (setme, 0);
+
+	if (end == pch)
 		retval = end==NULL ? FALSE : *end==delimiter;
+	else {
+		pan_g_string_append_len (setme, pch, end-pch);
+		retval = TRUE;
 	}
 
 	if (setme_next_token)
@@ -611,12 +617,9 @@ commatize_ulong (gulong    num,
 		 char    * setme)
 {
 	char buf[32], *src=buf;
-	int len;
-	int i;
+	int len = g_snprintf (buf, sizeof(buf), "%ld", num);
+	int i = len % 3;
 
-	sprintf (buf, "%ld", num);
-	len = strlen (buf);
-	i = len % 3;
 	for (;;)
 	{
 		while (*src && i--)
@@ -640,13 +643,13 @@ commatize_ulong (gulong    num,
 ****
 ***/
 
-gchar*
-pan_str_escape (const gchar * str)
+char*
+pan_str_escape (const char * str)
 {
 	gint size_needed;
-	gchar * retval;
-	gchar * out;
-	const gchar * in;
+	char * retval;
+	char * out;
+	const char * in;
 
 	/* sanity clause */
 	if (!is_nonempty_string (str))
@@ -681,13 +684,17 @@ pan_str_escape (const gchar * str)
 	return retval;
 }
 
-gchar*
-pan_str_unescape (const gchar * escaped)
+char*
+pan_str_unescape (const char * escaped)
 {
-	const gchar * src = escaped;
-	gchar * buf = g_new0 (gchar, strlen(escaped)+1);
-	gchar * tgt = buf;
+	const char * src;
+	char * buf;
+	char * tgt;
 
+	g_return_val_if_fail (escaped!=NULL, NULL);
+
+	src = escaped;
+	tgt = buf = g_new0 (char, strlen(src)+1);
 	while (*src)
 	{
 		if (*src!='&') { *tgt++ = *src++; }
@@ -712,3 +719,122 @@ pan_str_unescape (const gchar * escaped)
 
 	return buf;
 }
+
+/***
+****  This is taken from Sylpheed 0.7.0.
+***/
+
+static char*
+strstr_with_skip_quote (const char *haystack, const char *needle)
+{
+	register guint haystack_len, needle_len;
+	gboolean in_squote = FALSE, in_dquote = FALSE;
+
+	haystack_len = strlen(haystack);
+	needle_len   = strlen(needle);
+
+	if (haystack_len < needle_len || needle_len == 0)
+		return NULL;
+
+	while (haystack_len >= needle_len) {
+		if (!in_squote && !in_dquote &&
+		    !strncmp(haystack, needle, needle_len))
+			return (char *)haystack;
+
+		/* 'foo"bar"' -> foo"bar"
+		   "foo'bar'" -> foo'bar' */
+		if (*haystack == '\'') {
+			if (in_squote)
+				in_squote = FALSE;
+			else if (!in_dquote)
+				in_squote = TRUE;
+		} else if (*haystack == '\"') {
+			if (in_dquote)
+				in_dquote = FALSE;
+			else if (!in_squote)
+				in_dquote = TRUE;
+		}
+
+		haystack++;
+		haystack_len--;
+	}
+
+	return NULL;
+}
+
+/* this fuction was taken from gstrfuncs.c in glib. */
+static char **
+strsplit_with_quote (const char * str,
+                     const char * delim,
+                     int max_tokens)
+{
+	GSList *string_list = NULL, *slist;
+	char **str_array, *s;
+	guint i, n = 1;
+
+	g_return_val_if_fail(str != NULL, NULL);
+	g_return_val_if_fail(delim != NULL, NULL);
+
+	if (max_tokens < 1)
+		max_tokens = G_MAXINT;
+
+	s = strstr_with_skip_quote(str, delim);
+	if (s) {
+		guint delimiter_len = strlen(delim);
+
+		do {
+			guint len;
+			char *new_str;
+
+			len = s - str;
+			new_str = g_new(char, len + 1);
+			strncpy(new_str, str, len);
+			new_str[len] = 0;
+			string_list = g_slist_prepend(string_list, new_str);
+			n++;
+			str = s + delimiter_len;
+			s = strstr_with_skip_quote(str, delim);
+		} while (--max_tokens && s);
+	}
+
+	if (*str) {
+		n++;
+		string_list = g_slist_prepend(string_list, g_strdup(str));
+	}
+
+	str_array = g_new(char*, n);
+
+	i = n - 1;
+
+	str_array[i--] = NULL;
+	for (slist = string_list; slist; slist = slist->next)
+		str_array[i--] = slist->data;
+
+	g_slist_free(string_list);
+
+	return str_array;
+}
+
+char**
+strsplit_command_line_arguments (const char * command_line)
+{
+	int i;
+	char ** argv;
+
+	/* split the command line into arguments */
+	argv = strsplit_with_quote (command_line, " ", 0);
+
+	/* remove the quotes from quoted arguments */
+	for (i=0; argv[i]!=NULL; i++) {
+		char * str = argv[i];
+		if (str[0] == '\'' || str[0] == '\"') {
+			int len = strlen(str);
+			if (str[len - 1] == str[0]) {
+				str[len - 1] = '\0';
+				memmove(str, str + 1, len - 1);
+			}
+		}
+	}
+
+	return argv;
+}
