@@ -51,7 +51,7 @@ server_name_compare (gconstpointer a, gconstpointer b)
 {
 	const Server* sa = (const Server*) a;
 	const Server* sb = (const Server*) b;
-	return g_strcasecmp (sa->name, sb->name);
+	return g_strcasecmp (server_get_name (sa), server_get_name (sb));
 }
 
 /**
@@ -278,7 +278,7 @@ serverlist_get_named_server (const char* servername)
 
 	for (l=serverlist; retval==NULL && l!=NULL; l=l->next) {
 		Server * s = SERVER(l->data);
-		if (!pan_strcmp (s->name, servername))
+		if (!pan_strcmp (server_get_name (s), servername))
 			retval = s;
 	}
 
@@ -296,7 +296,7 @@ serverlist_get_named_folder (const gchar * foldername)
 			 
 	/* get the folder server... */
 	server = serverlist_get_named_server (INTERNAL_SERVER_NAME);
-	g_return_val_if_fail (server!=NULL, NULL);
+	g_return_val_if_fail (server_is_valid (server), NULL);
 					 
 	/* get the folder... */
 	return server_get_named_group (server, foldername);
@@ -320,7 +320,7 @@ get_default_server (void)
 	/* otherwise just use the first server */
 	for (l=serverlist; retval==NULL && l!=NULL; l=l->next) {
 		Server * s = SERVER(l->data);
-		if (strcmp (s->name, INTERNAL_SERVER_NAME))
+		if (strcmp (server_get_name (s), INTERNAL_SERVER_NAME))
 			retval = s;
 	}
 
@@ -351,7 +351,7 @@ serverlist_init (void)
 void
 serverlist_add_server (Server * server)
 {
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid (server));
 
 	server_init_servers ();
 
@@ -364,7 +364,7 @@ serverlist_add_server (Server * server)
 void
 serverlist_remove_server (Server * server)
 {
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid (server));
 
 	serverlist = g_slist_remove (serverlist, server);
 	/* might be best to just let this leak.
@@ -399,44 +399,45 @@ serverlist_save (void)
 	{
 		char val[128];
 		Server *s = SERVER(l->data);
+		const char * sn = server_get_name (s);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Address", s->name);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Address", sn);
 		pan_config_set_string (buf, s->address);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Port", s->name );
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Port", sn);
 		pan_config_set_int (buf, s->port);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Auth_Required", s->name);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Auth_Required", sn);
 		pan_config_set_int (buf, s->need_auth);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Generate_Msgid", s->name);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Generate_Msgid", sn);
 		pan_config_set_int (buf, s->gen_msgid);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Username", s->name);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Username", sn);
 		pan_config_set_string (buf, s->username);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Password", s->name);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Password", sn);
 		pan_config_set_string (buf, s->password);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Max_Connections", s->name);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Max_Connections", sn);
 		pan_config_set_int (buf, s->max_connections);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Idle_Timeout_Secs", s->name);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Idle_Timeout_Secs", sn);
 		pan_config_set_int (buf, s->idle_secs_before_timeout);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Online", s->name);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Online", sn);
 		pan_config_set_bool (buf, s->online);
 
-		g_snprintf (buf, sizeof(buf),"/Pan/%s/Last_Grouplist_Update", s->name);
+		g_snprintf (buf, sizeof(buf),"/Pan/%s/Last_Grouplist_Update", sn);
 		g_snprintf (val, sizeof(val), "%lu", s->last_newgroup_list_time);
 		pan_config_set_string (buf, val);
 
-		g_snprintf (buf, sizeof(buf), "/Pan/%s/Reserve_Reader_Connection", s->name);
+		g_snprintf (buf, sizeof(buf), "/Pan/%s/Reserve_Reader_Connection", sn);
 		pan_config_set_bool (buf, s->reserve_connection_for_bodies);
 
 		if (servers->len)
 			g_string_append_c (servers, ',');
-		g_string_append (servers, s->name);
+		g_string_append (servers, server_get_name (s));
 	}
 
 	pan_config_set_string ("/Pan/Servers/Names", servers->str);
