@@ -37,6 +37,7 @@
 #include <pan/globals.h>
 #include <pan/queue.h>
 #include <pan/task-headers.h>
+#include <pan/articlelist.h>
 #include <pan/util.h>
 
 #include <pan/dialogs/dialogs.h>
@@ -62,6 +63,15 @@ DownloadHeadersDialogStruct;
 **********************  BEGINNING OF SOURCE
 *********************/
 
+static void
+switch_to_group (gpointer callobj, gpointer callarg, gpointer client)
+{
+	Group * group = GROUP (client);
+
+	if (group->article_qty > 0)
+		articlelist_set_group_nolock (group);
+}
+
 static void
 download_headers_clicked_cb (GnomeDialog * dialog, int num, gpointer data)
 {
@@ -85,6 +95,12 @@ download_headers_clicked_cb (GnomeDialog * dialog, int num, gpointer data)
 			task = task_headers_new_sample (ui->group, sample_size, FALSE);
 		}
 
+		if (task != NULL)
+		{
+			pan_callback_add (TASK(task)->task_ran_callback, 
+				switch_to_group, ui->group);
+		}
+
 		if (task != NULL)
 		{
 			const gboolean download_bodies = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(ui->download_bodies_cb));
