@@ -31,6 +31,7 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 
+#include <pan/base/pan-config.h>
 #include <pan/base/serverlist.h>
 
 #include <pan/globals.h>
@@ -73,29 +74,10 @@ Druid;
 ****
 ***/
 
-static char*
-get_host_name (void)
-{
-	char *ptr;
-	char hostname[256] = { '\0' };
-
-	if ((gethostname(hostname, sizeof(hostname))) == -1) {
-		if ((ptr = getenv("HOST")) != NULL)
-			strncpy(hostname, ptr, MAXHOSTNAMELEN);
-		else if ((ptr = getenv("HOSTNAME")) != NULL)
-			strncpy(hostname, ptr, MAXHOSTNAMELEN);
-		else
-			hostname[0] = '\0';
-	}
-	hostname[255] = '\0';
-	ptr = strtok (hostname, ".");
-
-	return g_strdup (ptr);
-}
 
 /* Returns the fully qualified domain name */
 static char*
-get_fqdn (const char * host)
+get_fqdn (void)
 {
 	char name[512] = { '\0' };
 	char line[1025];
@@ -106,17 +88,10 @@ get_fqdn (const char * host)
 	struct hostent * hp;
 	struct in_addr in;
 
-	if (host) {
-		if (strchr(host, '.'))
-			return g_strdup(host);
-		strncpy(name, host, MAXHOSTNAMELEN);
-	}
-	else {
-		if (gethostname(name, MAXHOSTNAMELEN))
-			return NULL;
-	}
+	if (gethostname(name, MAXHOSTNAMELEN))
+		return NULL;
 
-	if (isdigit((int)*name)) {
+	if (isdigit((guchar)*name)) {
 		in.s_addr = inet_addr (name);
 		if ((hp = gethostbyaddr((char *) &in.s_addr, 4, AF_INET)))
 			in.s_addr = (*hp->h_addr);
@@ -160,18 +135,16 @@ get_fqdn (const char * host)
 static gchar*
 get_default_author_address (void)
 {
-	gchar * email = gnome_config_get_string ("/Pan/User/Email");
+	gchar * email = pan_config_get_string ("/Pan/User/Email");
 
 	if (email != NULL)
 		g_strstrip (email);
 
 	/* if no email in prefs, do the best we can */
 	if (!is_nonempty_string(email)) {
-		gchar * host = get_host_name();
-		gchar * fqdn = get_fqdn (host);
+		gchar * fqdn = get_fqdn ();
 		replace_gstr (&email,
 			g_strdup_printf ("%s@%s", g_get_user_name(), fqdn));
-		g_free (host);
 		g_free (fqdn);
 	}
 
@@ -216,12 +189,12 @@ druid_finish (GnomeDruidPage  * page,
 		server->max_connections = 4;
 		server->idle_secs_before_timeout = 180;
 		serverlist_add_server (server);
-		gnome_config_set_string ("Pan/State/Server", profile_name);
+		pan_config_set_string ("Pan/State/Server", profile_name);
 	}
 
 	/* save user stats */
-	gnome_config_set_string ("/Pan/Mail/smtp_port", smtp_port);
-	gnome_config_set_string ("/Pan/Mail/smtp_address", smtp_server);
+	pan_config_set_string ("/Pan/Mail/smtp_port", smtp_port);
+	pan_config_set_string ("/Pan/Mail/smtp_address", smtp_server);
 	mail_server_address = g_strdup (smtp_server);
 
 	id = identity_new ();
@@ -241,7 +214,7 @@ druid_finish (GnomeDruidPage  * page,
 	}
 
 	/* start Pan proper */
-	gnome_config_sync ();
+	pan_config_sync ();
 	gui_construct (NULL);
 	gtk_widget_show_all (Pan.window);
 	serverlist_set_active_server (server);
@@ -319,7 +292,6 @@ druid_email_page (Druid *ddata)
 	gchar * email;
 	gchar * smtp_server;
 	gchar * smtp_port;
-	gchar * host;
 	gchar * fqdn;
 
 	table = gtk_table_new (6, 2, FALSE);
@@ -358,8 +330,7 @@ druid_email_page (Druid *ddata)
 	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 4, 5, GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);
 
 	w = ddata->smtp_server = gtk_entry_new ();
-	host = get_host_name();
-	fqdn = get_fqdn (host);
+	fqdn = get_fqdn ();
 	smtp_server = g_strdup_printf ("%s", fqdn);
 	gtk_entry_set_text (GTK_ENTRY(w), smtp_server);
 	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 4, 5, GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);
@@ -376,7 +347,6 @@ druid_email_page (Druid *ddata)
 	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 5, 6, GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);
 
 	/* cleanup */
-	g_free (host);
 	g_free (fqdn);
 	g_free (email);
 	g_free (smtp_port);
@@ -394,6 +364,56 @@ password_needed_cb (GtkToggleButton * tb, Druid * druid)
 	gtk_widget_set_sensitive (druid->server_password_label, b);
 }
 
+static gchar*
+get_default_nntp_server (void)
+{
+	gchar * retval = NULL;
+
+	if (!retval) {
+		const gchar * cpch = g_getenv ("NNTPSERVER");
+		if (is_nonempty_string (cpch))
+			retval = g_strdup (cpch);
+	}
+
+	if (!retval) {
+		gchar * fqdn = get_fqdn ();
+		if (is_nonempty_string (fqdn)) {
+			const gchar * dot = strchr (fqdn, '.');
+			if (dot != NULL) {
+				retval = g_strdup_printf ("news.%s", dot+1);
+			}
+		}
+		g_free (fqdn);
+	}
+
+	return retval;
+}
+
+static gchar*
+get_default_profile_name (void)
+{
+	gchar * retval = NULL;
+
+	if (!retval) {
+		gchar * pch = get_default_nntp_server ();
+		if (is_nonempty_string (pch)) {
+			gchar * lastdot = strrchr (pch, '.');
+			if (lastdot != NULL)
+				*lastdot = '\0';
+			lastdot = strrchr (pch, '.');
+			if (lastdot != NULL)
+				retval = g_strdup (lastdot+1);
+		}
+		g_free (pch);
+	}
+
+	if (!retval) {
+		retval = g_strdup (_("default"));
+	}
+
+	return retval;
+}
+
 /*
  * Page #3:
  *  "Internet News Server"
@@ -403,7 +423,7 @@ druid_server_page (Druid *ddata)
 {
 	GtkWidget * w;
 	GtkWidget * table;
-	gchar * env_nntp_adr;
+	gchar * default_nntp_server;
 
 	table = gtk_table_new (7, 2, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
@@ -420,10 +440,12 @@ druid_server_page (Druid *ddata)
 	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 1, 2, GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);
 
 	w = ddata->server_address = gtk_entry_new();
-	if ((env_nntp_adr = g_getenv ("NNTPSERVER")) != NULL)
-		gtk_entry_set_text (GTK_ENTRY(w), env_nntp_adr);
+	default_nntp_server = get_default_nntp_server ();
+	if (is_nonempty_string (default_nntp_server))
+		gtk_entry_set_text (GTK_ENTRY(w), default_nntp_server);
 	else
 		gtk_entry_set_text (GTK_ENTRY(w), "news");
+	g_free (default_nntp_server);
 	gtk_table_attach (GTK_TABLE (table), w, 1, 2, 1, 2, GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);
 
 	w = gtk_label_new (_("Port:"));
@@ -472,6 +494,7 @@ druid_server_page (Druid *ddata)
 static GtkWidget *
 druid_profile_page (Druid *ddata)
 {
+	gchar * pch;
 	GtkWidget * w = NULL;
 	GtkWidget * table = NULL;
 
@@ -490,7 +513,9 @@ druid_profile_page (Druid *ddata)
 	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 1, 2, GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);
 
 	w = ddata->profile_name = gtk_entry_new();
-	gtk_entry_set_text (GTK_ENTRY(w), "Default");
+	pch = get_default_profile_name ();
+	gtk_entry_set_text (GTK_ENTRY(w), pch);
+	g_free (pch);
 	gtk_table_attach (GTK_TABLE(table), w, 1, 2, 1, 2, GTK_FILL, GTK_FILL, GNOME_PAD, GNOME_PAD_SMALL);
 
 	w = gtk_label_new (_("For example: UUNET Server"));
