@@ -21,8 +21,7 @@
 #ifndef __DIALOGS_H__
 #define __DIALOGS_H__
 
-#include <pan/group.h>
-#include <pan/rule.h>
+#include <pan/base/group.h>
 
 void dialog_about	        (GtkWidget * window);
 
@@ -36,6 +35,4 @@ void dialog_log_viewer	        (void);
 
 void dialog_rules               (void);
 
-void dialog_edit_rule           (Rule *);
-
 #endif
