@@ -21,16 +21,8 @@
 #ifndef __GUI_NOTEBOOK_H__
 #define __GUI_NOTEBOOK_H__
 
-/***
- *** global variables
- ***/
-extern GtkWidget *notebook;
-
-/***
- *** function prototypes
- ***/
 void gui_notebook_construct  (void);
-void gui_notebook_page_set (int page, GtkWidget * focus_item);
-
+int  gui_notebook_get_current_pane_nolock (void);
+void gui_notebook_page_set_nolock (int page, GtkWidget * focus_item);
 
 #endif /* __GUI_NOTEBOOK_H__ */
