@@ -21,9 +21,8 @@
 #ifndef __GUI_PANED_H__
 #define __GUI_PANED_H__
 
-
-int  gui_paned_get_current_pane         (void);
+int  gui_paned_get_current_pane_nolock  (void);
 void gui_paned_construct	        (void);
-void gui_paned_enter_clicked_nolock	(void);
+void gui_paned_page_set_nolock          (int page, GtkWidget * focus_item);
 
 #endif /* __GUI_PANED_H__ */
