@@ -392,6 +392,10 @@ static GnomeUIInfo navigate_menu[] =
 	                              N_("Previous Thread"),
 	                              articlelist_read_prev_thread,
 	                              GNOME_STOCK_MENU_BACK, 'V', GDK_CONTROL_MASK),
+	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("Previous _read article"),	
+	                              N_("Previous read article"),
+	                              articlelist_read_last_read,
+	                              GNOME_STOCK_MENU_BACK, 'V', GDK_SHIFT_MASK),
 	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("_Parent Article"),
 	                              N_("Parent Article"),
 	                              articlelist_read_parent,
@@ -1233,7 +1237,7 @@ gui_set_title_idle (gpointer data)
 	/* look for active server & group */
 	server = serverlist_get_active_server ();
 	if (server != NULL)
-		servername = server->name;
+		servername = server_get_name (server);
 	if (group != NULL)
 		groupname = group_get_readable_name (group);
 
@@ -1349,6 +1353,7 @@ update_menus_idle (gpointer data)
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* next thread */
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* prev */
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* prev thread */
+	gtk_widget_set_sensitive (menu[++i].widget, have_group && articlelist_has_prev_read()); /* prev read */
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* parent */
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* top */
 	++i;
@@ -1637,9 +1642,9 @@ gui_page_set_idle (gpointer page_gpointer)
 	switch (Pan.viewmode)
 	{
 		case GUI_PANED:
-			/* small tweak: it's not very useful to focus the message pane */
+			/* blah, users hated this.  remove after string thaw.
 			if (page==ARTICLELIST_PAGE || page==MESSAGE_PAGE)
-				focus_pane = gui_page_get_pane_nolock (ARTICLELIST_PAGE);
+				focus_pane = gui_page_get_pane_nolock (ARTICLELIST_PAGE); */
 		    	gui_paned_page_set_nolock (page, focus_pane);
 			break;
 
