@@ -22,11 +22,18 @@
 #define __MESSAGE_CHECK_UI_H__
 
 #include <glib.h>
-#include <pan/message-check.h>
+#include <pan/base/message-check.h>
+#include <gtk/gtkwindow.h>
 
-void check_article_and_prompt (const Article    * article_to_post,
-                               Server           * server_to_post_article,
-                               GPtrArray        * appendme_error_strings,
-                               GoodnessLevel    * setme_goodness_level);
+typedef void (*ArticleCheckedFunc)(Server          * server,
+                                   Article         * article,
+                                   GoodnessLevel     goodness,
+                                   gpointer          user_data);
+
+void check_article_and_prompt (GtkWidget           * window,
+                               Server              * server_to_post_article,
+                               Article             * article_to_post,
+                               ArticleCheckedFunc    checked_func,
+                               gpointer              checked_func_user_data);
 
 #endif
