@@ -40,6 +40,7 @@
 
 #include <pan/message-check-ui.h>
 #include <pan/message-send.h>
+#include <pan/globals.h>
 #include <pan/nntp.h>
 #include <pan/prefs.h>
 #include <pan/queue.h>
@@ -515,24 +516,12 @@ send_article_now (StatusItem           * item,
 *****
 ****/
 
-gint
-queue_article_for_posting (Server     * server,
-                           Article    * article,
-                           gboolean     check_article_first)
+static void
+queue_article_checked_cb (Server        * server,
+                          Article       * article,
+                          GoodnessLevel   goodness,
+                          gpointer        user_data)
 {
-	GPtrArray * errors;
-	GoodnessLevel goodness = OKAY;
-	debug_enter ("queue_article_for_posting");
-
-	/* sanity check */
-	g_return_val_if_fail (server!=NULL, 1);
-	g_return_val_if_fail (article_is_valid(article), 1);
-
-	/* if desired, make sure the article is okay. */
-       	errors = g_ptr_array_new ();
-	if (check_article_first)
-		check_article_and_prompt (article, server, errors, &goodness);
-
 	/* if the goodness is okay, queue a send task */
 	if (goodness == OKAY)
 	{
@@ -558,12 +547,22 @@ queue_article_for_posting (Server     * server,
 			article_get_extra_header(article,HEADER_NEWSGROUPS));
 		queue_add (task);
 	}
+}
 
-	/* cleanup */
-	pan_g_ptr_array_foreach (errors, (GFunc)g_free, NULL);
-	g_ptr_array_free (errors, TRUE);
-	debug_exit ("queue_article_for_posting");
-	return goodness==OKAY ? 0 : 1;
+void
+queue_article_for_posting (Server     * server,
+                           Article    * article,
+                           gboolean     check_article_first)
+{
+	/* sanity check */
+	g_return_if_fail (server_is_valid (server));
+	g_return_if_fail (article_is_valid(article));
+
+	/* if desired, make sure the article is okay. */
+	if (check_article_first)
+		check_article_and_prompt (Pan.window, server, article, queue_article_checked_cb, NULL);
+	else
+		queue_article_checked_cb (server, article, OKAY, NULL);
 }
 
 
