@@ -21,11 +21,11 @@
 #ifndef __MESSAGE_SEND_H__
 #define __MESSAGE_SEND_H__
 
-#include "server.h"
-#include "article.h"
 #include <glib.h>
+#include <pan/base/article.h>
+#include <pan/base/server.h>
 
-gint queue_article_for_posting (Server          * server,
+void queue_article_for_posting (Server          * server,
 		                Article         * article,
                                 gboolean          check_article_first);
 
