@@ -30,6 +30,7 @@
 
 #include <libgnomeui/libgnomeui.h>
 
+#include <pan/base/argset.h>
 #include <pan/base/debug.h>
 #include <pan/base/gnksa.h>
 #include <pan/base/log.h>
@@ -367,6 +368,7 @@ create_reply_body (Article * article)
 	marked = pch != NULL;
 	if (pch == NULL)
 		pch = article_get_body (article);
+	g_strchomp (pch);
 	if (is_nonempty_string(pch)) {
 		gchar ** sd = g_strsplit (pch, "\n", -1);
 		gint i;
@@ -389,6 +391,8 @@ create_reply_body (Article * article)
 		g_free (pch);
 	}
 
+	g_string_append (out, "\n \n");
+
 
 	/* return the result */
 	pch = out->str;
@@ -979,6 +983,7 @@ create_post_info_pane (Compose *mw)
 	 * ---- */
 	
 	headers = gtk_table_new (4, 2, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER(headers), GNOME_PAD);
 	gtk_table_set_row_spacings (GTK_TABLE(headers), GNOME_PAD_SMALL);
 	gtk_table_set_col_spacings (GTK_TABLE(headers), GNOME_PAD_SMALL);
 
@@ -1353,6 +1358,7 @@ remove_custom_headers (GtkWidget * header_text, Identity * id)
 		}
 	}
 	g_strfreev (lines);
+	g_free (text);
 
 	gtk_editable_delete_text (GTK_EDITABLE (header_text), 0, -1);
 	gtk_editable_insert_text (GTK_EDITABLE (header_text), new_body->str, 
@@ -1390,6 +1396,7 @@ add_custom_headers (GtkWidget * header_text, Identity * id)
 	                          new_body->len, &pos);
 
 	g_string_free (new_body, TRUE);
+	g_free (text);
 }
 
 /***
@@ -1727,8 +1734,7 @@ populate_post_info_pane (Compose *mw)
 				default_newsgroups = FALSE;
 
 				/* Followup-To: poster */
-				if (!g_strcasecmp ("poster", followup)) /* explicit reply by mail */
-				{
+				if (!g_strcasecmp ("poster", followup)) /* explicit reply by mail */ {
 					const gchar * replyto = article_get_extra_header (a, HEADER_REPLY_TO);
 
 					gnome_ok_dialog_parented (
@@ -1743,6 +1749,17 @@ populate_post_info_pane (Compose *mw)
 						gtk_entry_set_text (GTK_ENTRY(mw->users), author);
 					}
 				}
+				else
+				/* Does Followup-To: contain a valid email address ?    */
+				/* Son-of-1036 says this is invalid, but what the hell. */
+				if (gnksa_check_from (followup, FALSE) == GNKSA_OK) {
+
+					gnome_ok_dialog_parented (
+						_("``Followup-To:`` contains an email address: sending email to author."),
+						GTK_WINDOW(mw->window));
+
+					gtk_entry_set_text (GTK_ENTRY(mw->users), followup);	
+				}
 				else /* explicit followup groups */
 				{
 					gtk_entry_set_text (GTK_ENTRY(mw->newsgroups), followup);
@@ -1827,6 +1844,14 @@ body_activate_cb (GtkWidget * widget, gpointer data) {
 		send_now_cb (NULL, data);
 }
 
+static gboolean
+ignore_activation_keys (GtkWidget   * widget,
+                        GdkEventKey * event,
+                        gpointer      data)
+{
+	return TRUE;
+}
+
 static GtkWidget*
 create_body_pane_nolock (Compose * mw)
 {
@@ -1839,6 +1864,10 @@ create_body_pane_nolock (Compose * mw)
 	gtk_signal_connect (GTK_OBJECT(mw->body), "activate",
 	                    GTK_SIGNAL_FUNC (body_activate_cb), mw);
 
+	/* http://bugzilla.gnome.org/show_bug.cgi?id=72337 */
+	gtk_signal_connect_after (GTK_OBJECT (mw->body), "key_press_event",
+	                          GTK_SIGNAL_FUNC (ignore_activation_keys), mw);
+
 	/* set the text widget's properties */
 	pan_widget_set_font (GTK_WIDGET(mw->body), message_body_font_fixed);
 	style = gtk_widget_get_style(mw->body);
@@ -2054,7 +2083,7 @@ message_window_new (Article * article, ComposeType type)
 		gtk_widget_grab_focus (mw->subject);
 		pan_unlock ();
 
-		gtk_window_set_default_size (GTK_WINDOW(mw->window), 525, 575);
+		gtk_window_set_default_size (GTK_WINDOW(mw->window), 600, 575);
 		gui_restore_window_size (mw->window, "message_window");
 
 		pan_lock ();
@@ -2206,6 +2235,8 @@ message_window_get_reverse_path (const Compose* mw)
 	if (status == GNKSA_OK)
 		retval = g_strdup (address);
 
+	g_free (from);
+
 	return retval;
 }
 
@@ -2408,35 +2439,19 @@ typedef enum
 SendMode;
 
 static void
-post (Compose * mw, SendMode mode)
+post_checked_cb (Server        * server,
+                 Article       * article,
+                 GoodnessLevel   goodness,
+                 gpointer        user_data)
 {
-	Article * article;
-	Server * server;
-	GPtrArray * errors;
-	GoodnessLevel goodness;
-	const gchar * pch;
-	gchar * group_name;
+	ArgSet * argset = (ArgSet*)user_data;
+	Compose * mw = (Compose*) argset_get (argset, 0);
+	SendMode mode = GPOINTER_TO_INT(argset_get(argset, 1));
 
-	debug_enter ("post");
-
-       	server = serverlist_get_active_server ();
-       	errors = g_ptr_array_new ();
-
-	/* Get the article pointer, either through repopulating the
-	 * current article (if we're editing an existing message) or
-	 * by creating a new article (if this is a new message).
-	 */
-	if (mw->type == EDIT_ORIGINAL) {
-		article = mw->article;
-		populate_article_from_mw (article, mw);
-	} else {
-		article = build_article_from_window (mw);
-	}
-
-	/* if the article's okay, then send or queue */
-	check_article_and_prompt (article, server, errors, &goodness);
 	if (goodness == OKAY)
 	{
+		const char * pch;
+		char * group_name;
 		Group * sendlater = serverlist_get_named_folder (PAN_SENDLATER);
 
 		/* gotta ref the articles to make sure that, after sendlater
@@ -2465,9 +2480,7 @@ post (Compose * mw, SendMode mode)
 			g_strstrip (group_name);
 			group = server_get_named_group (server, group_name);
 			if (group!=NULL && !group_is_folder (group))
-			{
 				group_set_identity (group, mw->identity_name);
-			}
 
 			g_free (group_name);
 		}
@@ -2479,11 +2492,40 @@ post (Compose * mw, SendMode mode)
 	}
 
 	/* cleanup */
-	pan_g_ptr_array_foreach (errors, (GFunc)g_free, NULL);
-	g_ptr_array_free (errors, TRUE);
 	if (goodness!=OKAY && mw->type!=EDIT_ORIGINAL)
 		group_remove_article (serverlist_get_named_folder(PAN_SENDLATER), article);
 
+	argset_free (argset);
+}
+
+static void
+post (Compose * mw, SendMode mode)
+{
+	Article * article;
+	Server * server;
+
+	debug_enter ("post");
+
+       	server = serverlist_get_active_server ();
+
+	/* Get the article pointer, either through repopulating the
+	 * current article (if we're editing an existing message) or
+	 * by creating a new article (if this is a new message).
+	 */
+	if (mw->type == EDIT_ORIGINAL) {
+		article = mw->article;
+		populate_article_from_mw (article, mw);
+	} else {
+		article = build_article_from_window (mw);
+	}
+
+	/* if the article's okay, then send or queue */
+	check_article_and_prompt (mw->window,
+	                          server,
+	                          article,
+	                          post_checked_cb,
+	                          argset_new2 (mw, GINT_TO_POINTER(mode)));
+
 	debug_exit ("post");
 }
 
