@@ -21,6 +21,7 @@
 #include <config.h>
 
 #include <ctype.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -28,15 +29,17 @@
 
 #include <gmime/gmime-utils.h>
 
-#include <pan/acache.h>
-#include <pan/debug.h>
-#include <pan/log.h>
-#include <pan/nntp.h>
-#include <pan/pan-glib-extensions.h>
-#include <pan/pan-i18n.h>
+#include <pan/base/acache.h>
+#include <pan/base/debug.h>
+#include <pan/base/log.h>
+#include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-i18n.h>
+#include <pan/base/util-mime.h>
 
+#include <pan/nntp.h>
 #include <pan/prefs.h>
 #include <pan/task.h>
+#include <pan/auth-spa.h>
 
 gboolean online_prompt = TRUE;
 
@@ -54,6 +57,11 @@ enum
 	ERR_ACCESS				= 502
 };
 
+enum
+{
+	AUTH_UNKNOWN,
+	AUTH_MSN
+};
 
 /**
 ***
@@ -177,7 +185,6 @@ nntp_disconnect (StatusItem   * status,
 	debug_enter ("nntp_disconnect");
 
 	/* entry assertions */
-	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (socket!=NULL, TASK_FAIL_HOPELESS);
 
 	/* try to disconnect */
@@ -187,6 +194,14 @@ nntp_disconnect (StatusItem   * status,
 		return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 	}
 
+	/* Flush the socket. This ensures any remaining data (like TCP closing 
+	 * packets) are read between sending the NNTP command (which makes the 
+	 * server close the socket), and us closing our end of the socket.
+	 * This prevents the socket going into TIME_WAIT state (or worse, 
+	 * FIN_WAIT2)
+	 */
+	pan_socket_flush (socket);
+
 	/* disconnected successfully */
 	debug_exit ("nntp_disconnect");
 	return TASK_SUCCESS;
@@ -238,11 +253,87 @@ nntp_handshake (StatusItem  * status,
 	return val;
 }
 
+static gint
+nntp_spa_authenticate (StatusItem     * status,
+                        PanSocket      * sock)
+{
+	/* Authentication for SPA scheme (msn.com, and others) */
+	/* Added by Marc Prud'hommeaux <marc@apocalypse.org>, mostly */
+	/* taken from the ftechmail SPA authentication, which, in */
+	/* turn, was ripped out of the SAMBA project. */
+
+	gint val;
+	const gchar * response = NULL;
+	const gchar * response_trim = NULL;
+	gchar msgbuf[2048];
+	SPAAuthRequest   spa_request;
+	SPAAuthChallenge spa_challenge;
+	SPAAuthResponse  spa_response;
+
+	debug_enter ("nntp_spa_authenticate");
+
+	memset (msgbuf, 0, sizeof (msgbuf));
+
+	if (nntp_command (status, sock, &response, "AUTHINFO GENERIC MSN")
+	!= NEED_AUTHDATA) {
+		status_item_emit_error_va (status, _(
+			"Authentication failed: bad handshake for SPA password"));
+		return TASK_FAIL;
+	}
+	
+	spa_build_auth_request (&spa_request, sock->nntp_username, NULL);
+	spa_bits_to_base64 (msgbuf, (unsigned char*)&spa_request, 
+		spa_request_length (&spa_request));
+
+	val = nntp_command_va (status, sock, &response, 
+		"AUTHINFO GENERIC %s", msgbuf);
+	if (val != NEED_AUTHDATA) {
+		status_item_emit_error_va (status, _(
+			"Bad SPA handshake: %s"), response);
+		debug_exit ("nntp_spa_authenticate");
+		return TASK_FAIL;
+	}
+
+	if (!is_nonempty_string(sock->nntp_password)) {
+		if (status != NULL)
+			status_item_emit_error_va (status, _(
+				"Authentication failed: need a password"));
+		return TASK_FAIL;
+	}
+
+	/* response is now something like:					*/
+	/* 381 TlRMTVNTUAACAAAAGAAYACAAAAABAg [...]			*/
+	/* Trim off the first 4 bytes in our response.		*/
+	response_trim = response + 4;
+	spa_base64_to_bits ((unsigned char*)&spa_challenge, response_trim);
+
+	spa_build_auth_response (&spa_challenge, &spa_response, 
+		sock->nntp_username, sock->nntp_password);
+
+	spa_bits_to_base64 (msgbuf, (unsigned char*)&spa_response, 
+		spa_request_length (&spa_response));
+
+	val = nntp_command_va (status, sock, &response, 
+		"AUTHINFO GENERIC %s", msgbuf);
+
+	if (val != OK_AUTH) {
+		status_item_emit_error_va (status, _(
+			"Authentication failed: %s"), response);
+		debug_exit ("nntp_spa_authenticate");
+		return TASK_FAIL;
+	}
+
+	debug_exit ("nntp_spa_authenticate");
+	return TASK_SUCCESS;
+}
+
 gint
-nntp_authenticate (StatusItem     * status,
-                   PanSocket      * sock)
+nntp_authenticate_types (StatusItem     * status,
+                         PanSocket      * sock,
+                         gboolean         check_auth_types)
 {
 	gint val;
+	int authtype = AUTH_UNKNOWN;
 	const gchar * response = NULL;
 	debug_enter ("nntp_authenticate");
 
@@ -251,8 +342,54 @@ nntp_authenticate (StatusItem     * status,
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
 	g_return_val_if_fail (sock->nntp_username!=NULL, TASK_FAIL_HOPELESS);
 
+	/* check for authentication schemes */
+	/* see: http://www.mibsoftware.com/userkt/nntpext/0035.htm */
+	if (check_auth_types) {
+		val = nntp_command_va (status, sock, &response, "AUTHINFO GENERIC");
+		if (val==OK_AUTH) {
+			/* now we should be getting a list of valid authentication */
+			/* types terminated with a "." */
+			while (nntp_get_response (sock, &response) != -9999
+			&& response != NULL
+			&& pan_strcmp (response, ".\r\n")) {
+				/* supported authentication schemes follow */
+				if (!pan_strcmp (response, "MSN\r\n")) {
+					authtype = AUTH_MSN;
+				}
+				else {
+					status_item_emit_error_va (status, _(
+						"Unsupported authentication mechanism: %s"), response);
+				}
+			}
+	
+			switch (authtype)
+			{
+				case AUTH_MSN:
+					return nntp_spa_authenticate (status, sock);
+	
+				/* Unknown authentication type. We will fall through	*/
+				/* and try the normal authentication method anyway.		*/
+				case AUTH_UNKNOWN:
+				default:
+					status_item_emit_error_va (status, _(
+						"No supported authentication mechanism"));
+			}
+		}
+	}
+
 	/* set the username */
 	val = nntp_command_va (status, sock, &response, "AUTHINFO USER %s", sock->nntp_username);
+
+	/* On servers that require SPA authentication, sending			*/
+	/* "AUTHINFO USER ..." seems to result in a 502 error. If we	*/
+	/* have not yet tried other authentication methods, do so now.	*/
+	/* Ideally, we would not need to do this, but since some news`	*/
+	/* servers disconnect the user if they send "AUTHINFO GENERIC",	*/
+	/* we need to first try simple authentication.					*/
+	if (val==ERR_ACCESS && check_auth_types==FALSE) {
+		return nntp_authenticate_types (status, sock, TRUE);
+	}
+
 	if (val!=OK_AUTH && val!=NEED_AUTHDATA) {
 		if (status != NULL)
 			status_item_emit_error_va (status, _("Authentication failed: %s"), response);
@@ -280,6 +417,13 @@ nntp_authenticate (StatusItem     * status,
 	return TASK_SUCCESS;
 }
 
+gint
+nntp_authenticate (StatusItem     * status,
+                   PanSocket      * sock)
+{
+	return nntp_authenticate_types (status, sock, FALSE);
+}
+
 gint
 nntp_can_post (StatusItem   * status,
                PanSocket    * sock,
@@ -381,7 +525,7 @@ nntp_post (StatusItem    * status,
 		/* tell the server we want to post */
 		val = nntp_command (status, sock, &response, "POST");
 		if (val != 340) {
-			status_item_emit_error_va (status, "Posting failed: %s", response);
+			status_item_emit_error_va (status, _("Posting failed: %s"), response);
 			return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 		}
 
@@ -396,6 +540,7 @@ nntp_post (StatusItem    * status,
 	}
 
 	/* if we made it this far, we succeeded! */
+	debug_exit ("nntp_post");
 	return TASK_SUCCESS;
 }
 
@@ -422,27 +567,33 @@ nntp_noop (StatusItem    * status,
 **/
 
 static gboolean
-init_header_if_match (Article * a,
-                     const gchar * header,
-                     const gchar * header_key,
-                     const gchar * article_key)
+init_header_if_match (Article       * a,
+                      const gchar   * token,
+                      const gchar   * token_key,
+                      guint           token_key_len,
+                      const gchar   * header,
+                      HeaderAction    action)
 {
 	gboolean match = FALSE;
-	size_t len;
 
 	/* entry assertions */
 	g_return_val_if_fail (a!=NULL, FALSE);
-	g_return_val_if_fail (is_nonempty_string(header_key), FALSE);
-	g_return_val_if_fail (is_nonempty_string(article_key), FALSE);
-	g_return_val_if_fail (header!=NULL, FALSE);
+	g_return_val_if_fail (is_nonempty_string(token_key), FALSE);
+	g_return_val_if_fail (token_key_len>0, FALSE);
+	g_return_val_if_fail (is_nonempty_string(header), FALSE);
+	g_return_val_if_fail (token!=NULL, FALSE);
 
-	/* if this is the header we're looking for, keep it. */
-	len = strlen (header_key);
-	if (!strncmp (header, header_key, len))
+	/* if this is the token we're looking for, keep it. */
+	if (!strncmp (token, token_key, token_key_len))
 	{
-		gchar * b = g_mime_utils_8bit_header_decode (header+len);
-		article_init_header (a, article_key, b, DO_CHUNK);
-		g_free (b);
+		const gchar * val = token + token_key_len;
+		if (!pan_header_is_8bit_encoded (val))
+			article_init_header (a, header, val, action);
+		else {
+			gchar * b = g_mime_utils_8bit_header_decode ((const guchar*)val);
+			article_init_header (a, header, b, action);
+			g_free (b);
+		}
 		match = TRUE;
 	}
 
@@ -450,31 +601,32 @@ init_header_if_match (Article * a,
 }
 
 gint
-nntp_article_download (StatusItem      * status,
-                       PanSocket       * sock,
-                       Group           * group,
-                       Article         * article,
-                       const gboolean  * abort,
-                       gint              verbose,
-                       gboolean          checkout)
+nntp_article_download_to_gstr (StatusItem      * status,
+                               PanSocket       * sock,
+                               Group           * group,
+                               Article         * article,
+                               const gboolean  * abort,
+                               gint              verbose,
+                               gboolean          checkout,
+                               GString         * setme)
 {
 	const gchar * response;
 	gboolean getting_headers;
 	gint retval;
 	gint val;
-	GString * buf;
+	GString * line;
 	debug_enter ("nntp_article_download");
 
 	/* sanity clause */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL);
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
-	g_return_val_if_fail (group!=NULL, TASK_FAIL);
-	g_return_val_if_fail (article!=NULL, TASK_FAIL);
+	g_return_val_if_fail (group_is_valid(group), TASK_FAIL);
+	g_return_val_if_fail (article_is_valid(article), TASK_FAIL);
 	g_return_val_if_fail (abort!=NULL, TASK_FAIL);
 
 	/* let the user know what we're doing */
 	if (verbose & NNTP_VERBOSE_TEXT)
-		status_item_emit_status (status, article->subject);
+		status_item_emit_status (status, article_get_subject(article));
 
 	/* request the article from the server */
 	val = nntp_command_va (status, sock, &response, "ARTICLE %lu", article->number);
@@ -495,81 +647,115 @@ nntp_article_download (StatusItem      * status,
 
 	/* try to read the article... */
 	getting_headers = TRUE;
-	buf = g_string_sized_new (80*400); /* prealloc space for ~400 lines */
+	line = g_string_sized_new (80);
+
+	g_string_truncate (setme, 0);
 	retval = TASK_SUCCESS;
 	if (verbose & NNTP_VERBOSE_INIT_STEPS)
 		status_item_emit_init_steps (status, article->linecount);
 
 	for (;;)
 	{
-		gchar * line;
-		gchar * freeme;
+		const gchar * pch;
 
 		/* check for end cases: user abort, socket error, and success */
 		if (*abort || pan_socket_getline (sock, &response)) {
 			retval = TASK_FAIL;
 			break;
 		}
+
+		if (!is_nonempty_string(response))
+			continue;
+
 		if (!strncmp(response, ".\r\n", 3)) {
 			retval = TASK_SUCCESS;
 			break;
 		}
 
 		/* strip out the \r */
-		freeme = line = pan_substitute (response, "\r\n", "\n");
+		g_string_assign (line, response);
+		while ((pch = strstr (line->str, "\r\n")))
+			g_string_erase (line, pch-line->str, 1);
 
 		/* rfc 977: 2.4.1 */
-		if (line[0]=='.' && line[1]=='.')
-			++line;
+		if (line->len>=2 && line->str[0]=='.' && line->str[1]=='.')
+			g_string_erase (line, 0, 1);
 
 		/* save the line */
-		g_string_append (buf, line);
+		g_string_append (setme, line->str);
 		if (verbose & NNTP_VERBOSE_NEXT_STEP)
 			status_item_emit_next_step (status);
 
 		/* save the headers that we want to save */
 		if (getting_headers)
 		{
-			if (!strcmp(line, "\n")) /* header/body separator */
+			if (line->len==1 && !strcmp(line->str, "\n")) /* header/body separator */
 				getting_headers = FALSE;
 			else 
 			{
+				gint i;
 				Article * a = article;
 				gboolean matched = FALSE;
 
-				/* trim linefeed */
-				g_strstrip (line);
+				struct table {
+					const gchar * token;
+					guint token_len;
+					const gchar * header;
+					gint header_action;
+				};
+				static struct table tokens[] = {
+					{ "Newsgroups: ", 12, HEADER_NEWSGROUPS, DO_CHUNK_SHARE },
+					{ "Reply-To: ", 10, HEADER_REPLY_TO, DO_CHUNK_SHARE },
+					{ "Followup-To: ", 13, HEADER_FOLLOWUP_TO, DO_CHUNK_SHARE },
+					{ "Xref: ", 6, HEADER_XREF, DO_CHUNK },
+				};
+				static gint TOKEN_QTY = sizeof(tokens) / sizeof(tokens[0]);
 
-				/* check for headers to store in Article */
-				if (!matched)
-					matched = init_header_if_match (a, line,
-					                                "Newsgroups: ",
-					                                HEADER_NEWSGROUPS);
-				if (!matched)
-					matched = init_header_if_match (a, line,
-					                                "Reply-To: ",
-					                                HEADER_REPLY_TO);
-				if (!matched)
-					matched = init_header_if_match (a, line,
-					                                "Followup-To: ",
-					                                HEADER_FOLLOWUP_TO);
-				if (!matched)
-					matched = init_header_if_match (a, line,
-					                                "Xref: ",
-					                                HEADER_XREF);
+				/* trim linefeed */
+				g_strstrip (line->str);
+				line->len = strlen(line->str);
+
+				/* check the header */
+				for (i=0; !matched && i<TOKEN_QTY; ++i)
+					matched = init_header_if_match (a,
+					                                line->str,
+					                                tokens[i].token,
+					                                tokens[i].token_len,
+					                                tokens[i].header,
+					                                tokens[i].header_action);
 			}
 		}
-
-		g_free (freeme);
 	}
 
 	/* save the message */
 	if (retval == TASK_SUCCESS) {
-        	acache_set_message (article_get_message_id(article), buf->str, checkout);
+        	acache_set_message (article_get_message_id(article), setme->str, setme->len+1, checkout);
 		articles_remove_flag (&article, 1, STATE_DOWNLOAD_FLAGGED);
 	}
 
-	g_string_free (buf, TRUE);
+	g_string_free (line, TRUE);
+	debug_exit ("nntp_article_download");
+	return retval;
+}
+
+gint
+nntp_article_download (StatusItem      * status,
+                       PanSocket       * sock,
+                       Group           * group,
+                       Article         * article,
+                       const gboolean  * abort,
+                       gint              verbose,
+                       gboolean          checkout)
+{
+	GString * tmp;
+	gint retval;
+	debug_enter ("nntp_article_download");
+
+	/* download the message */
+ 	tmp = g_string_sized_new (80 * 400); /* preallocate space for ~400 lines */
+	retval = nntp_article_download_to_gstr (status, sock, group, article, abort, verbose, checkout, tmp);
+	g_string_free (tmp, TRUE);
+
 	debug_exit ("nntp_article_download");
 	return retval;
 }
@@ -636,12 +822,13 @@ nntp_download_headers (StatusItem       * status,
 	gboolean read_status = 0;
 	gint result;
 	gint val;
+	GString * buf;
 	debug_enter ("nntp_download_headers");
 
 	/* sanity checks */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
-	g_return_val_if_fail (group!=NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (group_is_valid(group), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (abort!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (is_nonempty_string(progress_fmt), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (setme!=NULL, TASK_FAIL_HOPELESS);
@@ -686,6 +873,7 @@ nntp_download_headers (StatusItem       * status,
 	***  Walk through all the articles headers that the server spits back
 	**/
 
+	buf = g_string_new (NULL);
 	pan_g_ptr_array_reserve (setme, setme->len + (high-low));
 	while (!*abort && !((read_status = pan_socket_getline (sock, &buffer))))
 	{
@@ -693,76 +881,90 @@ nntp_download_headers (StatusItem       * status,
 	        gint parts = 0;
 		const gchar * pch;
 		const gchar * s;
-		gchar * tmp;
 		Article * a;
+		const gchar * march = NULL;
+		const gchar * tok = NULL;
+		int tok_len = 0;
 
 		/* handle end-of-list */
 		if (!strncmp(buffer, ".\r\n", 3))
 			break;
 
 		/* create the article data */
-		pch = buffer;
 		a = article_new (group);
-		group_set_articles_dirty (group);
+		a->is_new = TRUE;
 
-		/* article number */
-		a->number = get_next_token_ulong (pch, '\t', &pch);
+		/* setup for parsing */
+		g_string_assign (buf, buffer);
+		march = buf->str;
 
-		/* subject */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		replace_gstr (&tmp, g_mime_utils_8bit_header_decode(tmp));
-		if (!is_nonempty_string(tmp))
-			replace_gstr (&tmp, g_strdup(_("No Subject")));
-		article_init_header (a, HEADER_SUBJECT, tmp, DO_CHUNK);
-		g_free (tmp);
+		/* get article number */
+		a->number = get_next_token_ulong (march, '\t', &march);
 
-		/* author */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		if (is_nonempty_string(tmp))
-			article_init_author_from_header (a, tmp);
-		g_free (tmp);
-
-		/* date */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		a->date = is_nonempty_string(tmp)
-			? g_mime_utils_header_decode_date (tmp, NULL)
-			: (time_t)0;
-		g_free (tmp);
-
-		/* message id */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		if (is_nonempty_string(tmp) && *tmp=='<')
-			article_init_header (a, HEADER_MESSAGE_ID, tmp, DO_CHUNK);
-		g_free (tmp);
-
-		/* references */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		if (is_nonempty_string(tmp) && *tmp=='<')
-			article_init_header (a, HEADER_REFERENCES, tmp, DO_CHUNK);
-		g_free (tmp);
-
-		/* unused field: size */
-		skip_next_token (pch, '\t', &pch);
-
-		/* linecount */
-		a->linecount = get_next_token_int(pch, '\t', &pch);
-
-		/* crosspost */
-		tmp = get_next_token_str (pch, '\t', &pch);
-		if (tmp==NULL || strncmp (tmp, "Xref: ", 6))
-			a->crosspost_qty = 1;
-		else
-		{
-			const gchar * foo;
-
-			a->crosspost_qty = 0;
-			for (foo=tmp+6; *foo; ++foo)
-				if (*foo==':')
-					++a->crosspost_qty;
-
-			article_init_header (a, HEADER_XREF, tmp+6, DO_CHUNK);
+		/* get subject */
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			if (!pan_header_is_8bit_encoded (tok))
+				article_init_header (a, HEADER_SUBJECT, tok, DO_CHUNK_SHARE);
+			else {
+				gchar * dec = g_mime_utils_8bit_header_decode ((const guchar*)tok);
+				article_init_header (a, HEADER_SUBJECT, dec, DO_CHUNK_SHARE);
+				g_free (dec);
+			}
+		}
+
+		/* get author */
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			if (is_nonempty_string(tok))
+				article_init_author_from_header (a, tok);
+		}
+
+		/* get date */
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			a->date = is_nonempty_string(tok)
+				? g_mime_utils_header_decode_date (tok, NULL)
+				: (time_t)0;
+		}
+
+		/* get message id */
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			if (is_nonempty_string(tok) && *tok=='<')
+				article_init_header (a, HEADER_MESSAGE_ID, tok, DO_CHUNK);
+		}
+
+		/* get references */
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
+			if (tok_len>0 ) {
+				((gchar*)tok)[tok_len] = '\0';
+				if (is_nonempty_string(tok) && *tok=='<')
+					article_init_header (a, HEADER_REFERENCES, tok, DO_CHUNK_SHARE);
+			}
+		}
+
+		/* get unused field: size */
+		skip_next_token (march, '\t', &march);
+
+		/* get linecount */
+		a->linecount = get_next_token_int (march, '\t', &march);
+
+		/* get crossref */
+		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
+			((gchar*)tok)[tok_len] = '\0';
+			if (tok_len==0 || strncmp(tok,"Xref: ", 6))
+				a->crosspost_qty = 1;
+			else {
+				const gchar * foo;
+				g_strstrip ((gchar*)tok); /* remove trailing linefeed */
+				a->crosspost_qty = 0;
+				for (foo=tok+6; *foo; ++foo)
+					if (*foo==':')
+						++a->crosspost_qty;
+				article_init_header (a, HEADER_XREF, tok+6, DO_CHUNK);
+			}
 		}
-		g_free (tmp);
 
 		/**
 		***  Validity Checking
@@ -770,8 +972,7 @@ nntp_download_headers (StatusItem       * status,
 
 		if (!article_is_valid (a)) {
 			status_item_emit_error_va (status, 
-				_("Corrupt header skipped - no Message Id: %s"), buffer);
-			pan_object_unref (PAN_OBJECT(a));
+				_("Corrupt header skipped: %s"), buffer);
 			continue;
 		}
 
@@ -786,11 +987,11 @@ nntp_download_headers (StatusItem       * status,
 		{
 			/* find the ']' of [n/N] */
 			--pch;
-			if ((pch[1]!=')' && pch[1]!=']') || !isdigit((int)*pch))
+			if ((pch[1]!=')' && pch[1]!=']') || !isdigit((guchar)*pch))
 				continue;
 
 			/* find the '/' of [n/N] */
-			while (s!=pch && isdigit((int)*pch))
+			while (s!=pch && isdigit((guchar)*pch))
 				--pch;
 			if (s==pch || (*pch!='/' && *pch!='|'))
 				continue;
@@ -800,7 +1001,7 @@ nntp_download_headers (StatusItem       * status,
 			--pch;
 
 			/* find the '[' of [n/N] */
-			while (s!=pch && isdigit((int)*pch))
+			while (s!=pch && isdigit((guchar)*pch))
 				--pch;
 			if (s==pch || (*pch!='(' && *pch!='[')) {
 				parts = 0;
@@ -818,14 +1019,14 @@ nntp_download_headers (StatusItem       * status,
 		   a single-part binary */
 		if (!parts
 			&& a->linecount>400
-			&& (((strstr(group->name,"binaries")
-					|| strstr(group->name,"fan")
-				        || strstr(group->name,"mag")
-					|| strstr(group->name,"sex")))
-				|| ((strstr(s,".jpg") || strstr(s,".JPG")
-					|| strstr(s,".jpeg") || strstr(s,".JPEG")
-					|| strstr(s,".gif") || strstr(s,".GIF")
-					|| strstr(s,".tiff") || strstr(s,".TIFF")))))
+			&& (((pan_strstr(group->name,"binaries")
+					|| pan_strstr(group->name,"fan")
+				        || pan_strstr(group->name,"mag")
+					|| pan_strstr(group->name,"sex")))
+				|| ((pan_strstr(s,".jpg") || pan_strstr(s,".JPG")
+					|| pan_strstr(s,".jpeg") || pan_strstr(s,".JPEG")
+					|| pan_strstr(s,".gif") || pan_strstr(s,".GIF")
+					|| pan_strstr(s,".tiff") || pan_strstr(s,".TIFF")))))
 			part = parts = 1;
 
 		/* Verify Multipart info */
@@ -844,9 +1045,10 @@ nntp_download_headers (StatusItem       * status,
 
 		/* update the count & progress ui */
 		status_item_emit_next_step (status);
-		if (!(setme->len % 29))
-			status_item_emit_status_va (status, progress_fmt, setme->len, (high-low));
+		if (!(setme->len % 50))
+			status_item_emit_status_va (status, progress_fmt, setme->len, (high-low+1));
 	}
+	g_string_free (buf, TRUE);
 
 	if (read_status!=0)
 		status_item_emit_error (status, sockread_err_msg);
@@ -877,23 +1079,48 @@ nntp_download_bodies (StatusItem       * status,
 	gint lines_total;
 	debug_enter ("nntp_download_bodies");
 
-	/* sanity checks */
-	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (group!=NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
-	g_return_val_if_fail (abort!=NULL, TASK_FAIL_HOPELESS);
+	/* sanity checks pt 1 */
 	g_return_val_if_fail (articles!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (index!=NULL, TASK_FAIL_HOPELESS);
 
+	/**
+	***  If we have the bodies, then just unflag 'em and maybe check them out
+	**/
+
+	for (i=*index; i<articles->len; ++i)
+		if (!article_has_body (ARTICLE(g_ptr_array_index(articles,i))))
+			break;
+
+	if (i==articles->len) {
+		const gint qty = articles->len - *index;
+		if (qty > 0) {
+			articles_remove_flag ((Article**)articles->pdata+*index, qty, STATE_DOWNLOAD_FLAGGED);
+			if (checkout) {
+				const gchar ** message_ids = alloca (sizeof(gchar*) * qty);
+				for (i=*index; i<articles->len; ++i)
+					message_ids[i] = article_get_message_id (ARTICLE(g_ptr_array_index(articles,i)));
+				acache_file_checkout (message_ids, qty);
+			}
+		}
+		return TASK_SUCCESS;
+	}
+
+	/* sanity checks pt 2 */
+	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (group_is_valid(group), TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
+	g_return_val_if_fail (abort!=NULL, TASK_FAIL_HOPELESS);
+
+	/**
+	***  Phooey, we have to download some of the bodies
+	**/
+
 	/* change to the group we want */
 	if (*abort)
 		return TASK_FAIL;
 	result = nntp_set_group (status, sock, group->name);
-	if (result != TASK_SUCCESS) {
-		const gchar * g = group_get_readable_name (group);
-		status_item_emit_error_va (status, _("Couldn't select group \"%s\""), g);
+	if (result != TASK_SUCCESS)
 		return result;
-	}
 
 	/* status item */
 	lines_so_far = lines_total = 0;
@@ -914,7 +1141,7 @@ nntp_download_bodies (StatusItem       * status,
 
 		status_item_emit_status_va (status, _("Getting %d of %d: \"%s\""),
 		                                    1+*index, articles->len,
-		                                    article->subject);
+		                                    article_get_subject(article));
 
 		/* download if we have to. */
 		pch = article_get_message_id (article);
@@ -954,8 +1181,9 @@ nntp_cancel (StatusItem     * status,
              PanSocket      * sock)
 {
 	GString * msg;
-	gchar * author;
 	const gchar * newsgroups;
+	gchar author[512];
+	gchar * author_encoded;
 	gint retval;
 	const gchar * msg_id = article_get_message_id (article);
 	debug_enter ("nntp_cancel");
@@ -963,9 +1191,9 @@ nntp_cancel (StatusItem     * status,
 	/* sanity checks */
 	g_return_val_if_fail (sock != NULL, TASK_FAIL);
 	g_return_val_if_fail (status != NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (article != NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (article_is_valid(article), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (is_nonempty_string(msg_id), TASK_FAIL_HOPELESS);
-	newsgroups = article_get_header (article, HEADER_NEWSGROUPS);
+	newsgroups = article_get_extra_header (article, HEADER_NEWSGROUPS);
 	g_return_val_if_fail (is_nonempty_string(newsgroups), TASK_FAIL_HOPELESS);
 
 	/* let the user know what we're doing */
@@ -973,9 +1201,9 @@ nntp_cancel (StatusItem     * status,
 
 	/* build the message to post */
 	msg = g_string_new (NULL);
-	author = article_get_author_str(article);
-        replace_gstr (&author, g_mime_utils_8bit_header_encode(author));
-	g_string_sprintfa (msg, "From: %s\r\n", author);
+	article_get_author_str (article, author, sizeof(author));
+	author_encoded = g_mime_utils_8bit_header_encode ((const guchar*)author);
+	g_string_sprintfa (msg, "From: %s\r\n", author_encoded);
 	g_string_sprintfa (msg, "Newsgroups: %s\r\n", newsgroups);
 	g_string_sprintfa (msg, "Subject: cancel %s\r\n", msg_id);
 	g_string_sprintfa (msg, "Control: cancel %s\r\n", msg_id);
@@ -986,7 +1214,7 @@ nntp_cancel (StatusItem     * status,
 	retval = nntp_post (status, sock, msg->str);
 
 	/* cleanup */
-	g_free (author);
+	g_free (author_encoded);
 	g_string_free (msg, TRUE);
 
 	debug_exit ("nntp_cancel");
