@@ -58,6 +58,10 @@ gint    nntp_handshake            (StatusItem           * status,
 gint    nntp_authenticate         (StatusItem           * status,
                                    PanSocket            * socket);
 
+gint    nntp_authenticate_types   (StatusItem           * status,
+                                   PanSocket            * socket,
+                                   gboolean              check_auth_types);
+
 gint    nntp_disconnect           (StatusItem           * status,
                                    PanSocket            * socket);
 
@@ -84,6 +88,15 @@ gint    nntp_get_group_info       (StatusItem           * status,
                                    gulong               * setme_high_num,
                                    const gboolean       * abort);
 
+gint    nntp_article_download_to_gstr    (StatusItem           * status,
+                                           PanSocket            * sock,
+                                           Group                * group,
+                                           Article              * article,
+                                           const gboolean       * abort_download,
+			                   gint                   nntp_verbosity,
+			                   gboolean               checkout,
+			                   GString              * setme);
+
 gint    nntp_article_download     (StatusItem           * status,
                                    PanSocket            * sock,
                                    Group                * group,
