@@ -25,7 +25,7 @@
 
 #include <glib.h>
 
-#include "task.h"
+#include <pan/task.h>
 
 
 
@@ -79,9 +79,12 @@ extern PanCallback *queue_max_tries_changed;
 **/
 
 /**
- * @return a GSList of Task* currently in the queue.
+ * @return a GPtrArray of Tasks.
+ * When done, the tasks must be unreffed and the GPtrArray should be freed:
+ *   pan_g_ptr_array_foreach (a, (GFunc)pan_object_unref, NULL);
+ *   g_ptr_array_free (a, TRUE);
  */
-extern const GSList* queue_get_tasks (void);
+extern GPtrArray* queue_get_tasks (void);
 
 /**
  * Add an item to the queue.  This increases the task's refcount by 1.
