@@ -282,11 +282,11 @@ create_save_attachment_as_dialog (Group * group,
 	pan_lock ();
 
 	/* create & parent the dialog */
-	dialog = gnome_dialog_new (_("Pan: Save Attachments As"),
+	dialog = gnome_dialog_new (_("Pan: Save Articles As"),
 				   GNOME_STOCK_BUTTON_OK,
 				   GNOME_STOCK_BUTTON_CANCEL,
 				   NULL);
-	gnome_dialog_set_default (GNOME_DIALOG(dialog), 1); /* cancel */
+	gnome_dialog_set_default (GNOME_DIALOG(dialog), 0); /* ok */
 
 	/* "Location" pane */
 	frame = gtk_frame_new (_("Location"));
@@ -464,14 +464,14 @@ create_save_attachment_as_dialog (Group * group,
 	vbox = gtk_vbox_new (TRUE, 0);
 
 	/* front of queue */
-	w = gtk_radio_button_new_with_label (NULL, _("Add Task to the Front of the Queue"));
+	w = gtk_radio_button_new_with_label (NULL, _("Add Task to the Front of the Task Manager"));
 	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), !back_of_queue);
 	data->front_of_queue_rb = w;
 
 	/* back of queue (default) */
 	w = gtk_radio_button_new_with_label_from_widget (
-		GTK_RADIO_BUTTON(w), _("Add Task to the Back of the Queue"));
+		GTK_RADIO_BUTTON(w), _("Add Task to the Back of the Task Manager"));
 	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), back_of_queue);
 	data->back_of_queue_rb = w;
