@@ -143,19 +143,20 @@ server_menu_update (void)
 	{
 		GnomeUIInfo * info = buf + widget_index;
 		Server * s = SERVER(g_ptr_array_index(servers,i));
+		gchar * sname = (gchar *) server_get_name (s);
 
 		/* skip 'invisible' servers used for folders, etc. */
-		if (!strncmp (s->name, INTERNAL_SERVER_NAME, strlen(INTERNAL_SERVER_NAME)))
+		if (!strncmp (sname, INTERNAL_SERVER_NAME, strlen(INTERNAL_SERVER_NAME)))
 			continue;
 
 		g_hash_table_insert (server_to_menubutton, s, info);
 		info->type = GNOME_APP_UI_ITEM;
-		info->label = s->name;
-		info->hint = s->name;
+		info->label = sname;
+		info->hint = sname;
 		info->moreinfo = server_selected_cb;
 		info->user_data = s;
 		if (new_first_server_name == NULL)
-			new_first_server_name = g_strdup (s->name);
+			new_first_server_name = g_strdup (sname);
 		if (s == cur_server)
 			sel_index = widget_index;
 
@@ -222,7 +223,7 @@ online_prompt_cb (gint reply,
 void
 server_prompt_to_go_online (Server* server)
 {
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid (server));
 
 	if (!server->online && !server->online_prompted)
 	{
@@ -231,7 +232,7 @@ server_prompt_to_go_online (Server* server)
 		server->online_prompted = TRUE;
 
 		question = g_strdup_printf (_("Go Online for Server \"%s\"?"),
-		                            server->name);
+		                            server_get_name (server));
 
 		pan_lock ();
 		gnome_question_dialog_parented (question,
