@@ -191,7 +191,7 @@ smtp_send_article (StatusItem    * item,
 	**/
 
 	/* who is this mail to? */
-	mail_to = article_get_header (article, PAN_MAIL_TO);
+	mail_to = article_get_extra_header (article, PAN_MAIL_TO);
 	pan_warn_if_fail (is_nonempty_string (mail_to));
 	if (!is_nonempty_string(mail_to)) {
 		status_item_emit_error (item, _("No Recipients!"));
@@ -201,7 +201,7 @@ smtp_send_article (StatusItem    * item,
 	/* who is this mail from?  (part 1) */
 	reverse_path = NULL;
 	if (okay) {
-		reverse_path = article_get_header (article, PAN_REVERSE_PATH);
+		reverse_path = article_get_extra_header (article, PAN_REVERSE_PATH);
 		if (!is_nonempty_string(reverse_path)) {
 			g_message (_("Can't parse the From: line; hope for the best."));
 			reverse_path = article->author_addr;
@@ -215,7 +215,8 @@ smtp_send_article (StatusItem    * item,
 	/* who is this mail from?  (part 2) */
 	from = NULL;
 	if (okay) {
-		gchar * author = article_get_author_str (article);
+		gchar author[512];
+		article_get_author_str (article, author, sizeof(author));
 		if (is_nonempty_string (author))
 			from = g_strdup (author);
 		if (from == NULL) {
@@ -227,7 +228,6 @@ smtp_send_article (StatusItem    * item,
 			status_item_emit_error(item, _("No Sender Specified!"));
 			okay = FALSE;
 		}
-		g_free (author);
 	}
 
 	/**
@@ -258,14 +258,15 @@ smtp_send_article (StatusItem    * item,
 			{
 				const gchar * fmt = "RCPT TO: <%s>\r\n";
 				const char * name = (const char*) g_ptr_array_index (to_array, i);
-				gchar * address;
-				gchar * realname;
+				gchar address[512];
+				gchar realname[512];
+
+				gnksa_do_check_from (name,
+				                     address, sizeof(address),
+				                     realname, sizeof(realname),
+				                     FALSE);
 
-				gnksa_do_check_from (name, &address, &realname, FALSE);
 				okay = !smtp_send_line_va (item, sock, fmt, address);
-
-				g_free (address);
-				g_free (realname);
 			}
 			if (okay)
 			{
@@ -306,9 +307,9 @@ smtp_send_article (StatusItem    * item,
 	*** send the message
 	**/
         
-	references = article_get_header (article, HEADER_REFERENCES);
+	references = article->references;
 
-	reply_to = article_get_header (article, HEADER_REPLY_TO);
+	reply_to = article_get_extra_header (article, HEADER_REPLY_TO);
 
 	subject = article_get_subject (article);
 
@@ -397,7 +398,8 @@ smtp_handshake (StatusItem   * item,
 
 	gethostname (my_name, sizeof(my_name));
 	hp = gethostbyname (my_name);
-	strcpy (my_name, hp->h_name);
+	if (hp != NULL)
+		strcpy (my_name, hp->h_name);
 
 	if (smtp_send_line_va (item, sock, "HELO %s\r\n", my_name))
 		return -1;
