@@ -21,8 +21,8 @@
 #ifndef __SMTP_H__
 #define __SMTP_H__
 
-#include "article.h"
-#include "status-item.h"
+#include <pan/base/article.h>
+#include <pan/base/status-item.h>
 
 /**
  * Opens a new smtp socket and sends the specified message out.
