@@ -146,7 +146,7 @@ task_grouplist_describe (const StatusItem* item)
 
 	return g_strdup_printf ( _("Getting %s from %s"),
 		action,
-		TASK_GROUPLIST(item)->server->name );
+		server_get_name (TASK_GROUPLIST(item)->server) );
 }
 
 static gint
@@ -159,7 +159,7 @@ task_grouplist_run (Task *item)
 	status_item_emit_status_va (STATUS_ITEM(item),
 		_("Got %u groups for \"%s\""),
 		item_g->groups->len,
-		server->name);
+		server_get_name (server));
 
 	/**
 	***  Try to get the groups from the server
@@ -187,7 +187,7 @@ task_grouplist_run (Task *item)
 	**/
 
 	status_item_emit_status_va (STATUS_ITEM(item),
-		_("Merging groups for \"%s\""), server->name);
+		_("Adding groups to \"%s\""), server_get_name (server));
 
 	/* for the next time we call GROUPLIST_NEW... */
 	server->last_newgroup_list_time = time(0);
@@ -238,7 +238,7 @@ nntp_grouplist_download_all (TaskGroupList * item)
 	status_item_emit_status_va (status,
 		_("Got %u groups for \"%s\""),
 		groups->len,
-		server->name);
+		server_get_name (server));
 
 	/**
 	***  Get the group list, if we haven't already
@@ -276,7 +276,6 @@ nntp_grouplist_download_all (TaskGroupList * item)
 			/* create a new group object */
 			group = group_new (server, name);
 			group->permission = *permission;
-			group_set_dirty (group); /* make sure it gets saved */
 			g_ptr_array_add (groups, group);
 
 			/* periodic status feedback */
@@ -286,7 +285,7 @@ nntp_grouplist_download_all (TaskGroupList * item)
 				commatize_ulong ((gulong)groups->len, buf);
 				status_item_emit_status_va (status,
 					_("Got %s groups for \"%s\""),
-					buf, server->name);
+					buf, server_get_name (server));
 			}
 
 			/* cleanup */
@@ -294,6 +293,10 @@ nntp_grouplist_download_all (TaskGroupList * item)
 			g_free (permission);
 		}
 
+		/* make sure the new groups get saved */
+		if (groups->len)
+			server_set_group_type_dirty (server, SERVER_GROUPS_ALL);
+
 		if (!read_err)
 			item->got_list = TRUE;
 
@@ -359,7 +362,7 @@ nntp_grouplist_download_all (TaskGroupList * item)
 			status_item_emit_status_va (
 				STATUS_ITEM(item),
 				_("Got %s descriptions for \"%s\""),
-				buf, server->name);
+				buf, server_get_name (server));
 		}
 
 		/* try to update the group's description field */
@@ -390,7 +393,7 @@ nntp_grouplist_download_all (TaskGroupList * item)
 
 	log_add_va (LOG_INFO, _("Got %d groups for server `%s'"),
 		groups->len,
-		server->name);
+		server_get_name (server));
 
 	return TASK_SUCCESS;
 }
@@ -417,7 +420,7 @@ nntp_grouplist_download_new (TaskGroupList * item)
 	Group * group = NULL;
 
 	status_item_emit_status_va (STATUS_ITEM(item),
-		_("Retrieving 0 new groups for \"%s\""), server->name);
+		_("Retrieving 0 new groups for \"%s\""), server_get_name (server));
 
 	/* get the date of the last grouplist update */
 	gmtime_r (&server->last_newgroup_list_time, &gmt);
@@ -465,7 +468,7 @@ nntp_grouplist_download_new (TaskGroupList * item)
 
 			status_item_emit_status_va (STATUS_ITEM(item),
 				_("Retrieving %d new groups for \"%s\""), 
-				count, server->name);
+				count, server_get_name (server));
 		}
 
 		if (*abort || pan_socket_getline (sock, &response)) {
@@ -476,7 +479,7 @@ nntp_grouplist_download_new (TaskGroupList * item)
 	log_add_va (LOG_INFO, _("Got %d new groups"), count);
 
 	status_item_emit_status_va (STATUS_ITEM(item),
-		_("Got %d news groups for \"%s\""), count, server->name);
+		_("Got %d news groups for \"%s\""), count, server_get_name (server));
 
 	return TASK_SUCCESS;
 }
