@@ -21,7 +21,7 @@
 #ifndef TASK_GROUPLIST__H
 #define TASK_GROUPLIST__H
 
-#include <pan/group.h>
+#include <pan/base/group.h>
 
 #include <pan/task.h>
 
@@ -48,7 +48,6 @@ typedef struct
 TaskGroupList;
 
 PanObject* task_grouplist_new (Server                  * server,
-                               gboolean                  high_priority,
                                GrouplistDownloadType     type);
 
 #endif
