@@ -33,6 +33,7 @@
 
 #include <pan/base/article.h>
 #include <pan/base/debug.h>
+#include <pan/base/file-headers.h>
 #include <pan/base/log.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-i18n.h>
@@ -123,7 +124,6 @@ real_task_headers_new (Group                * group,
 
 	g_return_val_if_fail (group_is_valid(group), NULL);
 	g_return_val_if_fail (!group_is_folder(group), NULL);
-	g_return_val_if_fail (group->server!=NULL, NULL);
 	g_return_val_if_fail (dl_type==HEADERS_ALL || dl_type==HEADERS_NEW || dl_type==HEADERS_SAMPLE, NULL);
 
        	item = g_new0 (TaskHeaders, 1);
@@ -215,7 +215,7 @@ task_headers_run (Task* task)
 
 	/* where to start downloading? */
 	if (task_h->download_type == HEADERS_NEW)
-		low_to_fetch = MAX (low_to_fetch, task_h->group->article_high);
+		low_to_fetch = MAX (low_to_fetch, task_h->group->article_high+1);
 
 	/* no articles to fetch */
 	if (!retval_set && (high_to_fetch==0 || low_to_fetch>high_to_fetch)) {
@@ -267,20 +267,49 @@ task_headers_run (Task* task)
 			retval = result;
 		}
 
+
+		if (1)
+		{
+			GPtrArray * a = group_get_article_array (task_h->group);
+
+			if (a->len && task_h->group->loaded_since_last_fetch)
+			{
+				if (a->len > 0)
+				{
+					if (articles->len != 0)
+						articles_set_new ((Article**)a->pdata, a->len, FALSE);
+					else {
+						GPtrArray * read = g_ptr_array_new ();
+						guint i;
+						for (i=0; i<a->len; ++i) {
+							Article * article = ARTICLE(g_ptr_array_index (a, i));
+							if (article_is_read (article))
+								g_ptr_array_add (read, article);
+						}
+						if (read->len > 0)
+							articles_set_new ((Article**)read->pdata, read->len, FALSE);
+						g_ptr_array_free (read, TRUE);
+					}
+				}
+			}
+
+			g_ptr_array_free (a, TRUE);
+			task_h->group->loaded_since_last_fetch = FALSE;
+		}
+
 		/* if we got new articles, process them */
 		if (articles->len != 0)
 		{
 			/* log */
 			log_add_va (LOG_INFO, _("Fetched %u headers for `%s'"),
 				articles->len,
-				group_get_readable_name(TASK_HEADERS(task)->group));
+				group_get_readable_name(task_h->group));
 
 			/* add the articles to the group */
 			group_mark_new_article_number (task_h->group, task_h->group->article_high);
 			group_add_articles_remove_unused (task_h->group, articles, status);
 			group_expire_articles_not_in_range (task_h->group, low_in_group, high_in_group);
 			group_set_article_range (task_h->group, low_in_group, high_in_group);
-			group_thread_if_needed (task_h->group, status);
 
 			/* apply the rules to the new headers */
 			rule_manager_process_incoming_articles (articles);
@@ -319,8 +348,11 @@ task_headers_run (Task* task)
 
 	/* cleanup */
 	status_item_emit_progress (status, 0);
-	if (reffed)
-		group_unref_articles (task_h->group, STATUS_ITEM(status));
+	if (reffed) {
+		server_save_grouplist_if_dirty (task_h->group->server, NULL);
+		file_headers_save (task_h->group, status);
+		group_unref_articles (task_h->group, status);
+	}
 
 	/* if we made it this far without fail, return success. */
 	if (!retval_set)
@@ -345,7 +377,6 @@ task_get_header_range (TaskHeaders   * task,
 {
 	const gboolean * const abort = &TASK(task)->hint_abort;
 	PanSocket *sock = TASK(task)->sock;
-	Server *server = TASK(task)->server;
 	Group *group = TASK_HEADERS(task)->group;
 	HeaderDownloadType dl_type = TASK_HEADERS(task)->download_type;
 	const char * progress_fmt;
@@ -359,7 +390,6 @@ task_get_header_range (TaskHeaders   * task,
 
 	/* sanity checks */
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (server!=NULL, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (group_is_valid(group), TASK_FAIL_HOPELESS);
 
 	/* change to the right group */
@@ -382,17 +412,18 @@ task_get_header_range (TaskHeaders   * task,
 	if (dl_type==HEADERS_NEW)
 	{
 		gulong p_last = group->article_high;
-		if (p_last!=0 && p_last>hi_article_in_group)
+		if (p_last!=0 && p_last>=hi_article_in_group)
 		{
 			status_item_emit_status_va (STATUS_ITEM(task),
 				_("No new articles in group \"%s\""),
 			        group_get_readable_name(group));
 		}
 
-		lo_article_to_fetch = p_last!=0 ? p_last : lo_article_in_group;
+		lo_article_to_fetch = p_last!=0 ? p_last+1 : lo_article_in_group;
 		hi_article_to_fetch = hi_article_in_group;
-		progress_fmt = _("Getting new Headers %lu of %lu");
+		progress_fmt = _("New %lu of %lu");
 	}
+
 	else if (dl_type==HEADERS_SAMPLE)
 	{
 		const int qty = MIN (total_in_group, task->sample_size);
@@ -400,13 +431,13 @@ task_get_header_range (TaskHeaders   * task,
 			STATUS_ITEM(task), _("Sampling %d articles"), qty);
 		lo_article_to_fetch = hi_article_in_group - qty;
 		hi_article_to_fetch = hi_article_in_group;
-		progress_fmt = _("Sampling Headers %lu of %lu");
+		progress_fmt = _("Sampling %lu of %lu");
 	}
 	else if (dl_type==HEADERS_ALL)
 	{
 		lo_article_to_fetch = lo_article_in_group;
 		hi_article_to_fetch = hi_article_in_group;
-		progress_fmt = _("Getting Headers %lu of %lu");
+		progress_fmt = _("All %lu of %lu");
 	}
 	else
 	{
@@ -427,7 +458,6 @@ task_get_header_range (TaskHeaders   * task,
 	return TASK_SUCCESS;
 }
 
-
 static gchar*
 task_headers_describe (const StatusItem* item)
 {
