@@ -374,7 +374,8 @@ get_secs_left (const Task * task)
 
 static void
 get_percent_str (const Task   * task,
-                 gchar        * buf)
+                 gchar        * buf,
+                 gint           buf_size)
 {
 	int percent = 0;
 
@@ -384,12 +385,13 @@ get_percent_str (const Task   * task,
 	if (percent == 0)
 		*buf = '\0';
 	else
-		sprintf (buf, "%d%%", percent);
+		g_snprintf (buf, buf_size, "%d%%", percent);
 }
 
 static void
 get_xfer_str (const Task    * task,
-              gchar         * buf)
+              gchar         * buf,
+              gint            buf_size)
 {
 	if (!task->sock)
 	{
@@ -402,13 +404,14 @@ get_xfer_str (const Task    * task,
 		const int m = (secs_left % 3600) / 60;
 		const int s = secs_left % 60;
 		const double KBps = pan_socket_get_xfer_rate_KBps (task->sock);
-		sprintf (buf, _("%.2f KB/s, %d:%02d:%02d remaining"), KBps, h, m, s);
+		g_snprintf (buf, buf_size, _("%.2f KB/s, %d:%02d:%02d remaining"), KBps, h, m, s);
 	}
 }
 
 static void
 get_status_str (const Task     * task,
-                gchar          * buf)
+                gchar          * buf,
+                gint             buf_size)
 {
 	const char* status_str;
 	QueueTaskStatus status = queue_get_task_status(task);
@@ -439,11 +442,11 @@ get_status_str (const Task     * task,
 	}
 
 	if (!task->server->online && status==QUEUE_TASK_STATUS_QUEUED)
-		sprintf (buf, _("Offline"));
+		g_snprintf (buf, buf_size, _("Offline"));
 	else if (!task->tries)
-		sprintf (buf, "%s", status_str);
+		g_snprintf (buf, buf_size, "%s", status_str);
 	else
-		sprintf (buf, _("%s (%d tries)"), status_str, task->tries);
+		g_snprintf (buf, buf_size, _("%s (%d tries)"), status_str, task->tries);
 }
 
 
@@ -466,9 +469,9 @@ add_task_to_list (ManagerUI     * ui,
 	text[3] = xferbuf;
 	text[4] = description;
 
-	get_xfer_str (task, xferbuf);
-	get_status_str (task, statusbuf);
-	get_percent_str (task, percentbuf);
+	get_xfer_str (task, xferbuf, sizeof(xferbuf));
+	get_status_str (task, statusbuf, sizeof(statusbuf));
+	get_percent_str (task, percentbuf, sizeof(percentbuf));
 
 	pan_lock ();
 	row = gtk_clist_insert (list, index, text);
@@ -601,7 +604,7 @@ gui_key_press_cb (GtkWidget      * widget,
 }
 
 static void
-update_titlebar_lock_unconditional (ManagerUI * ui)
+update_titlebar_lock (ManagerUI * ui)
 {
 	int running, queued, failed;
 	guint i;
@@ -634,9 +637,9 @@ update_titlebar_lock_unconditional (ManagerUI * ui)
 		pch = g_strdup_printf (_("Pan %s Task Manager"), VERSION);
 
 	/* show the titlebar */
-	pan_lock_unconditional ();
+	pan_lock ();
 	gtk_window_set_title (GTK_WINDOW(ui->window), pch);
-	pan_unlock_unconditional ();
+	pan_unlock ();
 	g_free (pch);
 
 	/* cleanup */
@@ -654,7 +657,7 @@ periodic_update_timer (gpointer user_data)
 	debug0 (DEBUG_QUEUE, "task manager periodic ui refresh");
 
 	/* update the ui */
-	pan_lock_unconditional ();
+	pan_lock ();
 	gtk_clist_freeze (clist);
 	for (row=0; row<clist->rows; ++row)
 	{
@@ -668,12 +671,12 @@ periodic_update_timer (gpointer user_data)
 			pan_object_ref (PAN_OBJECT(task));
 
 			/* have the index, now update it */
-			get_status_str (task, buf);
+			get_status_str (task, buf, sizeof(buf));
 			gtk_clist_set_text (clist, row, 0, buf);
-			get_percent_str (task, buf);
+			get_percent_str (task, buf, sizeof(buf));
 			gtk_clist_set_text (clist, row, 1, buf);
 			gtk_clist_set_text (clist, row, 2, task->server->name);
-			get_xfer_str (task, buf);
+			get_xfer_str (task, buf, sizeof(buf));
 			gtk_clist_set_text (clist, row, 3, buf);
 			pch = status_item_describe(STATUS_ITEM(task));
 			gtk_clist_set_text (clist, row, 4, pch);
@@ -685,10 +688,11 @@ periodic_update_timer (gpointer user_data)
 	}
 
 	gtk_clist_thaw (clist);
-	pan_unlock_unconditional ();
+	pan_unlock ();
 
-	if (ui->titlebar_needs_refresh) {
-		update_titlebar_lock_unconditional (ui);
+	if (ui->titlebar_needs_refresh)
+	{
+		update_titlebar_lock (ui);
 		ui->titlebar_needs_refresh = FALSE;
 	}
 
@@ -867,7 +871,7 @@ task_manager_build_ui (GtkWidget * window)
 	pan_callback_add (queue_task_moved, task_moved_cb, ui);
 
 	ui->titlebar_needs_refresh = TRUE;
-	ui->timeout_id = gtk_timeout_add (2000, periodic_update_timer, ui);
+	ui->timeout_id = pan_timeout_add (2000, periodic_update_timer, ui);
 	ui->top = vbox;
         gtk_signal_connect (GTK_OBJECT(ui->top), "destroy", ui_destroy_cb, ui);
 	return ui;
