@@ -23,8 +23,6 @@
 #include <ctype.h>
 #include <string.h>
 
-extern void* alloca (size_t);
-
 #include <glib.h>
 #include <libgnomeui/libgnomeui.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -32,11 +30,11 @@ extern void* alloca (size_t);
 #include <gmime/gmime.h>
 
 #include <pan/base/argset.h>
+#include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-object.h>
 #include <pan/base/pan-i18n.h>
-#include <pan/base/util-file.h>
 #include <pan/base/util-mime.h>
 #include <pan/base/util-wrap.h>
 
@@ -94,11 +92,34 @@ GdkColor text_quoted_color[3];
 GdkColor text_url_color;
 PanCallback * current_article_changed = NULL;
 
-static GnomeUIInfo text_pane_popup_uiinfo [];
 static GtkWidget * text_pane_popup_menu;
 
 void update_body_pane_nolock (GtkWidget*, const gchar*, gboolean);
 
+#define GNOMEUIINFO_ITEM_STOCK_ACCEL(label, tooltip, callback, stock_id, accel, modifier) \
+        { GNOME_APP_UI_ITEM, label, tooltip, (gpointer)callback, NULL, NULL, \
+		GNOME_APP_PIXMAP_STOCK, stock_id, accel, (GdkModifierType)modifier,  NULL }
+
+static GnomeUIInfo text_pane_popup_uiinfo [] =
+{
+	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("_Follow-Up to newsgroup"),
+	                              N_("Post a reply to the message on the news server."),
+	                              message_followup_window,
+	                              GNOME_STOCK_MENU_MAIL_RPL, 'F', 0),
+        GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("_Reply by E-mail"),
+	                              N_("Create a mail reply to the sender."),
+	                              message_reply_window,
+	                              GNOME_STOCK_MENU_MAIL_RPL, 'R', 0),
+	GNOMEUIINFO_ITEM_STOCK       (N_("Follow-Up _and Reply"),
+	                              N_("Send a reply both to the author in mail, and to the news server."),
+	                              message_followup_reply_window,
+	                              GNOME_STOCK_MENU_MAIL_RPL),
+	GNOMEUIINFO_ITEM_NONE        (N_("For_ward article by E-mail"),
+	                              N_("Forward article by E-mail"),
+	                              message_forward_window),
+	GNOMEUIINFO_END
+};
+
 
 /****
 *****
@@ -209,6 +230,7 @@ make_clickable_parts (GtkText      * text,
 	{
 		/* token */
 		const gchar * needle;
+		gint needle_len;
 
 		/* token search function */
 		gchar *(*search) (const gchar * haystack,
@@ -226,9 +248,9 @@ make_clickable_parts (GtkText      * text,
 	};
 
 	static struct table parser[] = {
-		{"http://",   pan_stristr,   get_uri_part,  make_uri_string},
-		{"https://",  pan_stristr,   get_uri_part,  make_uri_string},
-		{"ftp://",    pan_stristr,   get_uri_part,  make_uri_string}
+		{"http://",   7, pan_stristr,   get_uri_part,  make_uri_string},
+		{"https://",  8, pan_stristr,   get_uri_part,  make_uri_string},
+		{"ftp://",    6, pan_stristr,   get_uri_part,  make_uri_string}
 	};
 	const gint PARSE_ELEMS = sizeof(parser) / sizeof(parser[0]);
 
@@ -266,11 +288,11 @@ make_clickable_parts (GtkText      * text,
 
 			/* check if URI can be parsed */
 			if (parser[last_index].parse (linebuf, scanpos, &bp, &ep)
-			    && (ep-bp-1) > strlen(parser[last_index].needle)) {
+			    && (ep-bp-1) > parser[last_index].needle_len) {
 				ADD_TXT_POS (bp, ep, last_index);
 				walk = ep;
 			} else
-				walk = scanpos + strlen(parser[last_index].needle);
+				walk = scanpos + parser[last_index].needle_len;
 		}
 	}
 
@@ -307,35 +329,47 @@ make_clickable_parts (GtkText      * text,
 	}
 }
 
-static void
+static gboolean
 text_button_pressed (GtkWidget * text, GdkEventButton * event, TextView * view)
 {
-	g_return_if_fail (view!=NULL);
-	g_return_if_fail (view->uris!=NULL);
+	gboolean retval = FALSE;
 
-	if (event==NULL)
-	       return;
+	g_return_val_if_fail (view!=NULL, retval);
+	g_return_val_if_fail (view->uris!=NULL, retval);
+	g_return_val_if_fail (event!=NULL, retval);
 
-	if (event->button==2 || (event->button==1 && event->type==GDK_2BUTTON_PRESS))
+	if ((event->button==2 || event->button==1) && event->type==GDK_2BUTTON_PRESS)
 	{
+		static guint32 last_click;
 		guint i;
 		guint current_pos = GTK_EDITABLE(text)->current_pos;
 
-		for (i=0; i<view->uris->len; ++i)
+		if (event->time - last_click > 200)
 		{
-			const RemoteURI * uri = (const RemoteURI *) g_ptr_array_index (view->uris, i);
+			for (i=0; i<view->uris->len; ++i)
+			{
+				const RemoteURI * uri = (const RemoteURI *) g_ptr_array_index (view->uris, i);
 
-			if (uri->start<=current_pos && current_pos<uri->end)
-				pan_url_show (uri->uri);
+				if (uri->start<=current_pos && current_pos<uri->end)
+				{
+					pan_url_show (uri->uri);
+					gtk_signal_emit_stop_by_name (GTK_OBJECT(text), "button_press_event");
+					retval = TRUE;
+				}
+			}
 		}
+		last_click = event->time;
 	}
-	else if (event->button==3)
+	if (event->button==3 && event->type==GDK_BUTTON_PRESS)
 	{
 		gtk_menu_popup (GTK_MENU(text_pane_popup_menu),
-		                NULL, NULL, NULL, NULL,
-		                event->button, event->time);
-
+				NULL, NULL, NULL, NULL,
+				event->button, event->time);
+		gtk_signal_emit_stop_by_name (GTK_OBJECT(text), "button_press_event");
+		retval = TRUE;
 	}
+
+	return retval;
 }
 
 static GdkColor*
@@ -345,12 +379,12 @@ get_quote_color (const gchar * line)
 
 	if (is_nonempty_string(line))
 	{
-		gint depth = 0;
+		int depth = 0;
 
-		while (isspace((int)*line))
+		while (isspace((guchar)*line))
 			++line;
-		while (*line=='>' || isspace((int)*line)) {
-			if (*line=='>')
+		while (isspace((guchar)*line) || is_quote_char((guchar)*line)) {
+			if (is_quote_char((guchar)*line))
 				++depth;
 			++line;
 		}
@@ -520,7 +554,7 @@ text_set_raw_mainthread (gpointer data)
 	if (text == NULL)
 		text = g_strdup (" ");
 
-	pan_lock_unconditional ();
+	pan_lock ();
 	if (GTK_IS_OBJECT(Pan.text))
 	{
 		gint position = 0;
@@ -532,10 +566,10 @@ text_set_raw_mainthread (gpointer data)
 		if (is_nonempty_string(text))
 			gtk_editable_insert_text (editable_w, text, strlen(text), &position);
 		gtk_text_thaw (text_w);
-		g_free (text);
 	}
-	pan_unlock_unconditional ();
+	pan_unlock ();
 
+	g_free (text);
 	debug_exit ("text_set_raw_mainthread");
 	return 0;
 }
@@ -543,9 +577,7 @@ text_set_raw_mainthread (gpointer data)
 void
 text_set_raw (const gchar * text)
 {
-	pan_lock ();
-	run_in_main_thread_nolock (text_set_raw_mainthread, g_strdup(text));
-	pan_unlock ();
+	gui_queue_add (text_set_raw_mainthread, g_strdup(text));
 }
 
 
@@ -561,7 +593,7 @@ text_set_raw (const gchar * text)
  * Used for displaying attached pictures inline.
  */
 static GtkWidget*
-get_gtk_pixmap_from_gmime_part (const GMimePart * part)
+get_gtk_pixmap_from_gmime_part (GMimePart * part)
 {
 	guint len;
 	const gchar * content;
@@ -570,13 +602,24 @@ get_gtk_pixmap_from_gmime_part (const GMimePart * part)
 	GdkBitmap * mask = NULL;
 	GtkWidget * w = NULL;
 	GdkPixbufLoader * l = NULL;
+	YencInfo * yenc;
 
 	/* create the loader */
 	l = gdk_pixbuf_loader_new ();
 
+	/* FIXME: this is a hack to get single-part yenc messages working.
+	   Ideally some pan-gmime mediator should have decoded this already */
+	yenc = GMIME_OBJECT(part)->user_data;
+	if (yenc!= NULL && part->content!=NULL && part->content->stream!=NULL) {
+		GMimeStream * stream = g_mime_stream_filter_new_with_stream (part->content->stream);
+		g_mime_stream_filter_add (GMIME_STREAM_FILTER(stream),
+		                          g_mime_filter_yenc_new (GMIME_FILTER_YENC_DIRECTION_DECODE));
+		g_mime_data_wrapper_set_stream (part->content, stream);
+	}
+
 	/* create the pixbuf */
 	content = g_mime_part_get_content (part, &len);
-	gdk_pixbuf_loader_write (l, content, len);
+	gdk_pixbuf_loader_write (l, (const guchar*)content, len);
 	pixbuf = gdk_pixbuf_loader_get_pixbuf (l);
 	content = NULL;
 
@@ -637,7 +680,7 @@ extract_text_cb (GMimePart * part, gpointer data)
 		if (is_nonempty_string(pch))
 		{
 			if (rot13_enabled)
-				replace_gstr (&pch, rot13(pch));
+				rot13_inplace (pch);
                 	if (text_get_wrap())
 				replace_gstr (&pch, fill_body(pch,wrap_column));      
 		
@@ -665,8 +708,8 @@ set_text_from_article_nolock (const Article * article)
 	body = g_string_new (NULL);
 
 	/* headers */
-	if (current_mm!=NULL && text_get_show_all_headers()) {
-		gchar * pch = g_mime_message_get_headers (current_mm);
+	if (article!=NULL && text_get_show_all_headers()) {
+		gchar * pch = article_get_headers (article);
 		g_string_append (body, pch);
 		g_string_append_c (body, '\n');
 		g_free (pch);
@@ -676,13 +719,19 @@ set_text_from_article_nolock (const Article * article)
 	if (current_mm!=NULL)
 		g_mime_message_foreach_part (current_mm, extract_text_cb, body);
 
-	/* build the attachments table at the end of the body string */
+	/**
+	***  Build the attachments table at the end of the body string
+	**/
+
 	if (current_mm!=NULL)
 		mp = current_mm->mime_part;
-	if (mp!=NULL && mp->children!=NULL) {
+	if (mp!=NULL && mp->children!=NULL)
+	{
 		GList * l;
 		g_string_sprintfa (body, "\n\n");
-		for (l=mp->children; l!=NULL; l=l->next) {
+
+		for (l=mp->children; l!=NULL; l=l->next)
+		{
 			GMimePart * tmp = (GMimePart*) l->data;
 			gchar * content_type;
 			const gchar * filename;
@@ -702,8 +751,12 @@ set_text_from_article_nolock (const Article * article)
 		}
 	}
 
-	/* update the widgets */
-	if (1) {
+	/**
+	***  Update the widgets
+	**/
+
+	if (1)
+	{
 		gint pix_qty = 0;
 		gboolean pix_before;
 		gboolean need_child_change;
@@ -820,10 +873,9 @@ get_current_article (void)
 	return current_article;
 }
 
-static gint
-set_current_article_mainthread (gpointer data)
+static int
+set_current_article_impl (Article * a, const char * text)
 {
-	Article * a = ARTICLE(data);
 	Article * old_article;
 	GMimeMessage * old_mm;
 
@@ -837,53 +889,117 @@ set_current_article_mainthread (gpointer data)
 	old_article = current_article;
 	current_article = a;
 	current_mm = NULL;
-	if (a != NULL)
-	{
-		gchar * raw = article_get_message (a);
-		current_mm = pan_g_mime_parser_construct_message (raw);
-		g_free (raw);
-	}
+
+	/**
+	***  If we have a new article, parse it
+	**/
+
+	if (text != NULL)
+		current_mm = pan_g_mime_parser_construct_message (&text, 1);
+
+	/**
+	***  Fire an event that the current article has changed
+	**/
+
 	pan_callback_call (current_article_changed, old_article, current_article);
 
+	/**
+	***  Clean out the old article
+	**/
+
 	if (old_mm != NULL)
 	{
-		g_mime_message_destroy (old_mm);
+		g_mime_object_unref (GMIME_OBJECT(old_mm));
 	}
 	if (old_article != NULL)
 	{
 		group_unref_articles (old_article->group, NULL);
 	}
+
+	/**
+	***  Set up the new article
+	**/
+
 	if (current_article != NULL)
 	{
-		pan_lock_unconditional ();
+		pan_lock ();
 		set_text_from_article_nolock (current_article);
-		pan_unlock_unconditional ();
+		pan_unlock ();
+
+		articles_set_read (&current_article, 1, TRUE);
 	}
 	else
 	{
-		pan_lock_unconditional ();
+		pan_lock ();
 		gui_headers_set_default_nolock (header_table, NULL);
 		set_text_from_article_nolock (NULL);
-		pan_unlock_unconditional ();
+		pan_unlock ();
 	}
 
 	debug_exit ("set_current_article_mainthread");
 	return 0;
 }
 
+static gint
+set_current_article_mainthread (gpointer data)
+{
+	gchar * text = NULL;
+	Article * a = ARTICLE(data);
+
+	if (a != NULL)
+		text = article_get_message (a);
+
+	set_current_article_impl (a, text);	
+
+	g_free (text);
+	return 0;
+}
+
+static gint
+set_current_article_mem_mainthread (gpointer data)
+{
+	ArgSet * argset;
+	Article * a;
+	gchar * text;
+	debug_enter ("set_current_article_mem_mainthread");
+
+	/* let set_current_article_impl() to do the work */
+	argset = (ArgSet*) data;
+	a = ARTICLE (argset_get (argset, 0));
+	text = (gchar*) argset_get (argset, 1);
+	set_current_article_impl (a, text);
+
+	/* cleanup */
+	g_free (text);
+	argset_free (argset);
+	debug_exit ("set_current_article_mem_mainthread");
+	return 0;
+
+	
+}
+
 static void
 set_current_article (Article * article)
 {
 	debug_enter ("set_current_article");
 
+	/* sanity clause */
+	if (article!=NULL)
+		g_return_if_fail (article_is_valid(article));
+
 	still_on_welcome = FALSE;
 
-	if (article!=NULL && article_is_valid(article))
+	/**
+	***  Ref the articles s.t. we know that this article will
+	***  still be loaded when set_current_article_mainthread runs,
+	***  then push the GUI work over into the main thread.
+	**/
+
+	if (article!=NULL)
 		group_ref_articles (article->group, NULL);
 
-	pan_lock ();
-	run_in_main_thread_nolock (set_current_article_mainthread, article);
-	pan_unlock ();
+	gui_queue_add (set_current_article_mainthread, article);
+
 
 	debug_enter ("set_current_article");
 }
@@ -903,7 +1019,7 @@ text_clear_nolock (void)
 
 	/* clear out old */
 	if (old_mm != NULL)
-		g_mime_message_destroy (old_mm);
+		g_mime_object_unref (GMIME_OBJECT(old_mm));
 	if (old_article != NULL)
 		group_unref_articles (old_article->group, NULL);
 
@@ -926,6 +1042,26 @@ text_refresh (void)
 	}
 }
 
+void
+text_set_from_mem_article (Article * article, const gchar * text)
+{
+	debug_enter ("text_set_from_mem_article");
+
+	g_return_if_fail (article!=NULL);
+	g_return_if_fail (text!=NULL);
+
+	still_on_welcome = FALSE;
+
+	if (article!=NULL)
+		group_ref_articles (article->group, NULL);
+
+	gui_queue_add (set_current_article_mem_mainthread, argset_new2 (article, g_strdup(text)));
+
+	gui_page_set (MESSAGE_PAGE);
+
+	debug_exit ("text_set_from_mem_article");
+}
+
 void
 text_set_from_cached_article (Article * article)
 {
@@ -933,9 +1069,8 @@ text_set_from_cached_article (Article * article)
 
 	g_return_if_fail (article!=NULL);
 	g_return_if_fail (article_has_body(article));
-
 	set_current_article (article);
-	gui_page_set (MESSAGE_PAGE, Pan.text);
+	gui_page_set (MESSAGE_PAGE);
 
 	debug_exit ("text_set_from_cached_article");
 }
@@ -990,7 +1125,7 @@ group_articles_removed_cb (gpointer call_obj, gpointer call_arg, gpointer client
 	/* sanity checks */
 	g_return_if_fail (group!=NULL);
 	g_return_if_fail (removed!=NULL);
-	g_return_if_fail (removed->len>0);
+	g_return_if_fail (removed->len>0u);
 
 	/* unset the current article if we need to */
 	if (current_article!=NULL)
@@ -1049,7 +1184,8 @@ realize_text_cb (GtkWidget *text, gpointer data)
         pan_widget_set_font (GTK_WIDGET(Pan.text), fontname());
 
 	/* show the welcome message */
-	update_body_pane_nolock (Pan.text, welcome, FALSE);
+	if (still_on_welcome)
+		update_body_pane_nolock (Pan.text, welcome, FALSE);
 
 	/* this funtion only needs to be called once */
 	gtk_signal_disconnect_by_func (GTK_OBJECT(text), GTK_SIGNAL_FUNC(realize_text_cb), NULL);
@@ -1092,11 +1228,9 @@ text_create (void)
         gtk_box_pack_start (GTK_BOX(text_box), scrolled_window, TRUE, TRUE, 0); 
 
 	pan_callback_add (group_get_articles_removed_callback(),
-	                  group_articles_removed_cb,
-	                  NULL);
+	                  group_articles_removed_cb, NULL);
 	pan_callback_add (articlelist_get_group_changed_callback(),
-	                  articlelist_group_changed_cb,
-			  NULL);
+	                  articlelist_group_changed_cb, NULL);
 
 	/* create view object */
 	view = g_new0 (TextView, 1);
@@ -1289,27 +1423,4 @@ text_select_all (void)
 	debug_exit ("text_select_all");
 }
 
-#define GNOMEUIINFO_ITEM_STOCK_ACCEL(label, tooltip, callback, stock_id, accel, modifier) \
-        { GNOME_APP_UI_ITEM, label, tooltip, (gpointer)callback, NULL, NULL, \
-		GNOME_APP_PIXMAP_STOCK, stock_id, accel, (GdkModifierType)modifier,  NULL }
-
-
-static GnomeUIInfo text_pane_popup_uiinfo [] =
-{
-	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("_Follow-Up to newsgroup"),
-	                              N_("Post a reply to the message on the news server."),
-	                              message_followup_window,
-	                              GNOME_STOCK_MENU_MAIL_RPL, 'F', 0),
-        GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("_Reply by E-mail"),
-	                              N_("Create a mail reply to the sender."),
-	                              message_reply_window,
-	                              GNOME_STOCK_MENU_MAIL_RPL, 'R', 0),
-	GNOMEUIINFO_ITEM_STOCK       (N_("Follow-Up _and Reply"),
-	                              N_("Send a reply both to the author in mail, and to the news server."),
-	                              message_followup_reply_window,
-	                              GNOME_STOCK_MENU_MAIL_RPL),
-	GNOMEUIINFO_ITEM_NONE        (N_("For_ward article by E-mail"),
-	                              N_("Forward article by E-mail"),
-	                              message_forward_window)
-};
 
