@@ -173,7 +173,7 @@ thread_init_module (void)
 ************/
 
 gint
-thread_get_state (const Article * a)
+pan_thread_get_state (const Article * a)
 {
 	gint retval = THREAD_NOTHING;
 
@@ -203,16 +203,6 @@ thread_get_state (const Article * a)
 				g_free (id);
 			}
 		}
-
-		/* Maybe this Article has a broken References: header.
-		   Thanks to Adrian von Bidder for suggesting this.
-		   People posting broken References: headers should be flogged. */
-		if (retval==THREAD_NOTHING && a->parent!=NULL) {
-			Article * root = a->parent;
-			while (root->parent != NULL)
-				root = root->parent;
-			retval = thread_get_state (root);
-		}
 	}
 
 
@@ -220,7 +210,7 @@ thread_get_state (const Article * a)
 }
 
 void
-thread_set_state (const Article * a, int state)
+pan_thread_set_state (const Article * a, int state)
 {
 	gchar * id;
 
