@@ -25,6 +25,7 @@
 
 #include <gnome.h>
 
+#include <pan/base/argset.h>
 #include <pan/base/debug.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/log.h>
@@ -87,184 +88,123 @@ pan_error_dialog (const char *format, ...)
 	g_free (str);
 }
 
+/***
+****
+****  IDLE FUNC
+****
+***/
 
+/**
+ * Thus spoke the GTK FAQ: ``Callbacks require a bit of attention.
+ * Callbacks from GTK+ (signals) are made within the GTK+ lock. However
+ * callbacks from GLib (timeouts, IO callbacks, and idle functions) are
+ * made outside of the GTK+ lock. So, within a signal handler you do not
+ * need to call gdk_threads_enter(), but within the other types of
+ * callbacks, you do.''
+ *
+ * pan_timeout_add() and run_in_main_thread_idle() are wrappers around
+ * glib-level callbacks that ensure that pan_lock() works properly
+ * to give them a gdk_threads_enter() lock.
+ */
 
-static pthread_t has_lock_thr = 0;
+static gboolean main_thread_is_in_glib_callback = FALSE;
 
-void
-pan_lock_unconditional_from (const gchar * file, int line)
+static gint
+pan_timeout_wrapper (gpointer data)
 {
-	static const gchar * last_file = NULL;
-	static int last_line = -1;
-	pthread_t thr = pthread_self();
+	ArgSet * argset = (ArgSet*) data;
+	GtkFunction func = (GtkFunction) argset_get (argset, 0);
+	gpointer user_data = (gpointer) argset_get (argset, 1);
+	gint retval;
 
-	if (thr == has_lock_thr)
-	{
-		g_warning (_("ERROR thread %lu attempted double lock! %s:%d"),
-			(gulong)thr, file, line);
-		g_warning (_("ERROR last lock from %s:%d"), last_file, last_line);
-		pan_warn_if_reached ();
-	}
+	/* sanity clause */
+	pan_warn_if_fail (pthread_self() == Pan.main_t);
 
-	gdk_threads_enter();
-	last_file = file;
-	last_line = line;
-	has_lock_thr = thr;
-	debug3 (DEBUG_LOCK,"thread %lu entered gdk_threads from %s %d", thr, file, line);
+	/* call the timer func */
+	main_thread_is_in_glib_callback = TRUE;
+	retval = (*func)(user_data);
+	main_thread_is_in_glib_callback = FALSE;
+
+	/* cleanup */
+	if (!retval)
+		argset_free (argset);
+	return retval;
 }
-
-void
-pan_lock_from (const gchar * file, int line)
+guint
+pan_timeout_add (guint32 interval, GtkFunction func, gpointer arg)
 {
-	pthread_t thr = pthread_self();
-
-	if (thr==Pan.main_t && !has_lock_thr)
-	{
-		debug3 (DEBUG_LOCK,"mainthread %lu attempted lock from %s %d", thr, file, line);
-	}
-	else
-	{
-		pan_lock_unconditional_from (file, line);
-	}
-}
-
-void
-pan_unlock_unconditional_from (const gchar * file, int line)
-{
-	has_lock_thr = (pthread_t)0;
-	gdk_threads_leave();
-	debug3 (DEBUG_LOCK,"thread %lu left gdk_threads from %s %d", pthread_self(), file, line);
-}
-
-void
-pan_unlock_from (const gchar* file, int line)
-{
-	pthread_t thr = pthread_self();
-
-	if (thr==Pan.main_t && !has_lock_thr)
-	{
-		debug3 (DEBUG_LOCK,"mainthread %lu attempted unlock from %s %d", thr, file, line);
-	}
-	else
-	{
-		pan_unlock_unconditional_from (file, line);
-	}
+	return gtk_timeout_add (interval, pan_timeout_wrapper, argset_new2 (func, arg));
 }
 
 /***
 ****
-****  LINKIFY TEXT
+****  LOCKING
 ****
 ***/
 
-/* This is pretty much taken straight from
-   slrn's art.c, written by John E. Davis */
-static const gchar*
-find_url (const gchar  * in,
-          guint        * setme_url_len)
+static pthread_t has_lock_thr = 0;
+
+void
+pan_lock_from (const gchar * file, const gchar * func, int line)
 {
-	const gchar * ptr;
+	static const gchar * last_file = NULL;
+	static const gchar * last_func = NULL;
+	static int last_line = -1;
+	const pthread_t thr = pthread_self();
 
-	while ((ptr = pan_strstr (in, "://")) != NULL)
+	/**
+	 * If pan_lock() is called from the main thread while it has a GUI lock
+	 * (typically from a gtk signals, like a button press signal etc.)
+	 * then we don't need to lock.
+	 *
+	 * However if pan_lock() is called from a worker thread, or the main
+	 * thread inside a glib idle function (via run_in_main_thread_nolock())
+	 * then we _do_ need to obtain a gtk lock.
+	 */
+	if (thr==Pan.main_t && !main_thread_is_in_glib_callback)
 	{
-		const gchar * begin;
-		const gchar * end;
-		gboolean again;
-
-		/* walk back to the beginning of the word */
-		begin = ptr;
-		while ((begin>in) && isalpha((int)begin[-1]))
-			--begin;
-
-		/* all registered and reserved scheme names are >= 3 chars long */
-		if (begin+3 > ptr) {
-			in = ptr + 3;
-			continue;
-		}
-
-		end = ptr;
-		again = TRUE;
-		while (again)
-		{
-			gchar ch = *end;
-
-			switch (ch)
-			{
-				case ',':
-				case ';':
-				case '.':
-					ch = *++end;
-					if (isspace((int)ch)) {
-						--end;
-						again = FALSE;
-					}
-					break;
-
-				case ' ':
-				case '\t':
-				case '\n':
-				case '\"':
-				case '}':
-				case '{':
-				case ')':
-				case '>':
-				case '<':
-				case 0:
-					again = FALSE;
-					break;
-
-				default:
-					++end;
-					break;
-			}
-		}
-
-		*setme_url_len = (guint)(end - begin);
-		if (*setme_url_len < 7)
-			continue;
-
-		if (!strncmp (end-3, "://", 3))
-			continue;
-
-		return begin;
+		debug4 (DEBUG_LOCK,"mainthread %lu attempted unnecessary lock from %s:%d (%s)", thr, file, line, func);
 	}
+	else if (thr == has_lock_thr)
+	{
+		g_error (_("thread %lu attempted double lock!\nfirst lock was in %s:%d (%s),\nnow trying for another one from %s:%d (%s)"),
+			(gulong)thr,
+			last_file, last_line, last_func,
+			file, line, func);
+	}
+	else
+	{
+	       	/* if (thr==Pan.main_t && main_thread_is_in_glib_callback)
+			odebug3 ("idle func %s:%d (%s) getting a pan lock", file, line, func);*/
 
-	return NULL;
+		gdk_threads_enter();
+		last_file = file;
+		last_func = func;
+		last_line = line;
+		has_lock_thr = thr;
+		debug3 (DEBUG_LOCK,"thread %lu entered gdk_threads from %s %d", thr, file, line);
+	}
 }
 
-gchar*
-linkify_text (const gchar * text)
+void
+pan_unlock_from (const gchar* file, const gchar * func, int line)
 {
-	guint len = 0;
-	gchar * retval;
-	GString * str;
-	const gchar * prev = text;
-	const gchar * pch;
-	debug_enter ("linkify_text");
+	const pthread_t thr = pthread_self();
 
-       	str = g_string_new (NULL);
-	while ((pch = find_url(prev,&len)) != NULL)
+	if (thr==Pan.main_t && !has_lock_thr)
 	{
-		gchar * url = g_strndup (pch, len);
-		if (pch != prev)
-			pan_g_string_append_len (str, prev, (guint)(pch-prev));
-
-		g_string_append (str, "<a href=\"");
-		g_string_append (str, url);
-		g_string_append (str, "\">");
-		g_string_append (str, url);
-		g_string_append (str, "</a>");
-
-		prev = pch + len;
-		g_free (url);
+		debug4 (DEBUG_LOCK,"mainthread %lu attempted unneccessary unlock from %s:%d (%s)", (gulong)thr, file, line, func);
+	}
+	else if (has_lock_thr != thr)
+	{
+		g_error (_("thread %lu attempted to remove a lock it didn't have from %s:%d (%s)"), (gulong)thr, file, line, func);
+	}
+	else
+	{
+		has_lock_thr = (pthread_t)0;
+		gdk_threads_leave();
+		debug4 (DEBUG_LOCK,"thread %lu left gdk_threads from %s:%d (%s)", pthread_self(), file, line, func);
 	}
-	g_string_append (str, prev);
-
-	retval = str->str;
-	g_string_free (str, FALSE);
-
-	debug_exit ("linkify_text");
-	return retval;
 }
 
 /***
@@ -392,7 +332,7 @@ pan_url_show (const gchar * url)
 		if (is_nonempty_string(browser)) {
 			gint i;
 			gint result = -1;
-			gchar ** tries = g_strsplit (browser, ":", -1);
+			gchar ** tries = strsplit_command_line_arguments (browser);
 			for (i=0; result==-1 && tries[i]!=NULL; ++i) {
 				g_message ("trying [%s]", tries[i]);
 				result = try_to_execute_url (tries[i], url);
