@@ -33,7 +33,7 @@ int main (void)
 	if (1) {
 		gchar * data_dir = g_strdup_printf ("%s/data/", getenv("PWD"));
 		gchar * download_dir = g_strdup_printf ("%s/data/save/", getenv("PWD"));
-		base_prefs_init (data_dir, download_dir, 1000, FALSE, FALSE);
+		base_prefs_init (data_dir, download_dir, ">", 1000, FALSE, FALSE);
 		g_free (data_dir);
 		g_free (download_dir);
 	}
