@@ -99,12 +99,14 @@ init_pan (void)
 	base = g_strdup_printf ("%s/", base!=NULL ? base : getenv("PWD"));
 	data_dir = g_strdup_printf ("%s/data/", base);
 	download_dir = g_strdup ("/tmp/");
-	base_prefs_init (data_dir, download_dir, 100, FALSE, FALSE);
+	base_prefs_init (data_dir, download_dir, ">", 100, FALSE, FALSE);
 
 	g_free (data_dir);
 	g_free (download_dir);
 }
 
+extern int server_saving_enabled;
+
 int
 main (void)
 {
@@ -115,6 +117,7 @@ main (void)
 
 	g_thread_init (NULL);
 	init_pan ();
+	server_saving_enabled = FALSE;
 
 	/* init newsrc */
        	n = newsrc_new (NULL, 0, 1000);
@@ -322,6 +325,14 @@ main (void)
 	if (strcmp(newsrc_get_read_str(n),"17577-17578,17581-17582,17589-17590")) return num;
 	g_message ("PASS newsrc_mark_range %d", num);
 
+	/* Jiri Kubicek reported this one against 0.11.2.90 */
+	newsrc_init (n, "", 1, 10);
+	newsrc_mark_range (n, 1, 4, TRUE);
+	newsrc_mark_range (n, 7, 9, TRUE);
+	newsrc_mark_range (n, 3, 4, FALSE);
+	if (strcmp(newsrc_get_read_str(n),"1-2,7-9")) return num;
+	g_message ("PASS mark-unread-range on border of read block");
+
 	/* newsrc_set_group_range
 	 * note that after a mark all, the lower bound is 0.
 	 * this follows newsrc conventions. */
